-------------------------------------------------------------------------------------
-- Title: Mik's Scrolling Battle Text Options
-- Author: Mik
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------
-- Local Constants.
-------------------------------------------------------------------------------------

-- The name of the options frame.
local OPTIONS_FRAME_NAME = "MSBTFrameOptions";

-- The name of the font settings frame.
local FONT_SETTINGS_FRAME_NAME = "MSBTFontSettingsFrame";

-- The name of the scroll area frames.
local SCROLL_AREA_MOVER_FRAME_NAME = "MSBTScrollAreaMoverFrame";


-- The min and max font sizes allowed.
local MIN_FONT_SIZE = 12;
local MAX_FONT_SIZE = 32;

-- The min and max animation steps.
local MIN_ANIMATION_STEP = 1;
local MAX_ANIMATION_STEP = 5;

-- The min and max scroll heights.
local MIN_SCROLL_HEIGHT = 100;
local MAX_SCROLL_HEIGHT = 300;

-- The size of an event settings line.
local EVENT_ITEM_HEIGHT = 25;

-- The number of events to show per page.
local EVENT_ITEMS_PER_PAGE = 8;

-- Tab indices.
local TAB_INDEX_GENERAL		= 1;
local TAB_INDEX_INCOMING	= 2;
local TAB_INDEX_OUTGOING	= 3;
local TAB_INDEX_NOTIFICATION	= 4;


-- Dropdown indices.
local DROPDOWN_INDEX_PROFILE					= 1;
local DROPDOWN_INDEX_INCOMING_TEXT_ALIGN			= 2;
local DROPDOWN_INDEX_INCOMING_ANIMATION_STYLE 		= 3;
local DROPDOWN_INDEX_INCOMING_SCROLL_DIRECTION 		= 4;
local DROPDOWN_INDEX_OUTGOING_TEXT_ALIGN			= 5;
local DROPDOWN_INDEX_OUTGOING_ANIMATION_STYLE 		= 6
local DROPDOWN_INDEX_OUTGOING_SCROLL_DIRECTION 		= 7;
local DROPDOWN_INDEX_NOTIFICATION_TEXT_ALIGN		= 8;
local DROPDOWN_INDEX_NOTIFICATION_ANIMATION_STYLE	= 9;
local DROPDOWN_INDEX_NOTIFICATION_SCROLL_DIRECTION 	= 10;
local DROPDOWN_INDEX_FONT					= 11;
local DROPDOWN_INDEX_FONT_OUTLINE				= 12;
local DROPDOWN_INDEX_FONT_CRIT				= 13;
local DROPDOWN_INDEX_FONT_OUTLINE_CRIT			= 14;


-- Checkbox indices.
local CHECKBOX_INDEX_DISABLE_MSBT				= 1;
local CHECKBOX_INDEX_SHOW_PARTIAL_EFFECTS			= 2;
local CHECKBOX_INDEX_USE_STICKY_CRITS			= 3;
local CHECKBOX_INDEX_SHOW_OVERHEALS				= 4;
local CHECKBOX_INDEX_SHOW_GAME_DAMAGE			= 5;
local CHECKBOX_INDEX_SHOW_INCOMING_EVENTS			= 6;
local CHECKBOX_INDEX_SHOW_OUTGOING_EVENTS			= 7;
local CHECKBOX_INDEX_SHOW_NOTIFICATION_EVENTS		= 8;
local CHECKBOX_INDEX_INHERIT_FONT_SIZE			= 9;
local CHECKBOX_INDEX_INHERIT_FONT_SIZE_CRIT		= 10;
local CHECKBOX_INDEX_INCOMING_EVENTS_SHOW_START		= 11;
local CHECKBOX_INDEX_OUTGOING_EVENTS_SHOW_START		= 19;
local CHECKBOX_INDEX_OUTGOING_EVENTS_SHOW_START		= 27;
local CHECKBOX_INDEX_NOTIFICATION_EVENTS_STICKY_START	= 35;

-- Slider indices.
local SLIDER_INDEX_INCOMING_SCROLL_HEIGHT		= 1;
local SLIDER_INDEX_OUTGOING_SCROLL_HEIGHT		= 2;
local SLIDER_INDEX_NOTIFICATION_SCROLL_HEIGHT	= 3;
local SLIDER_INDEX_ANIMATION_STEP			= 4;
local SLIDER_INDEX_FONT_SIZE				= 5;
local SLIDER_INDEX_FONT_SIZE_CRIT			= 6;


-- Button indices.
local BUTTON_INDEX_EDIT_MASTER_FONT_SETTINGS		= 1;
local BUTTON_INDEX_EDIT_INCOMING_FONT_SETTINGS		= 2;
local BUTTON_INDEX_EDIT_OUTGOING_FONT_SETTINGS		= 3;
local BUTTON_INDEX_EDIT_NOTIFICATION_FONT_SETTINGS	= 4;
local BUTTON_INDEX_LOAD_PROFILE				= 5;
local BUTTON_INDEX_DELETE_PROFILE				= 6;
local BUTTON_INDEX_RESET_PROFILE				= 7;
local BUTTON_INDEX_CREATE_PROFILE				= 8;
local BUTTON_INDEX_MOVE_INCOMING_SCROLL_AREA		= 9;
local BUTTON_INDEX_MOVE_OUTGOING_SCROLL_AREA		= 10;
local BUTTON_INDEX_MOVE_NOTIFICATION_SCROLL_AREA	= 11;
local BUTTON_INDEX_SCROLL_AREA_MOVER_PREVIEW		= 12;
local BUTTON_INDEX_SCROLL_AREA_MOVER_SAVE			= 13;
local BUTTON_INDEX_SCROLL_AREA_MOVER_CANCEL		= 14;
local BUTTON_INDEX_INCOMING_EVENTS_START			= 15;
local BUTTON_INDEX_OUTGOING_EVENTS_START			= 23;
local BUTTON_INDEX_NOTIFICATION_EVENTS_START		= 31;


-- Editbox indices.
local EDITBOX_INDEX_NEW_PROFILE_NAME	= 1;
local EDITBOX_INDEX_EVENT_MESSAGE		= 2;

-- Colorswatch indices.
local COLORSWATCH_INDEX_INCOMING_EVENTS_START		= 1;
local COLORSWATCH_INDEX_OUTGOING_EVENTS_START		= 9;
local COLORSWATCH_INDEX_NOTIFICATION_EVENTS_START	= 17;


-- Frame indices.
local FRAME_INDEX_INCOMING_SCROLL_AREA_MOVER		= 1;
local FRAME_INDEX_OUTGOING_SCROLL_AREA_MOVER		= 2;
local FRAME_INDEX_NOTIFICATION_SCROLL_AREA_MOVER	= 3;


-------------------------------------------------------------------------------------
-- Local variables.
-------------------------------------------------------------------------------------

local interfaceData = {};

local availableFontsInherit;
local availableOutlinesInherit;


-------------------------------------------------------------------------------------
-- Core event handlers.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Called when the options frame is loaded.
-- **********************************************************************************
function MikSBTOpt.OnLoad()
 -- Register for the ADDON_LOADED event.
 this:RegisterEvent("ADDON_LOADED");
end


-- **********************************************************************************
-- Called when the events we registered for occur.
-- **********************************************************************************
function MikSBTOpt.OnEvent()
 -- When an addon is loaded.
 if (event == "ADDON_LOADED") then
  -- Make sure it's this addon.
  if (arg1 == string.gsub(MikSBT.MOD_NAME .. "Options", "%s", "")) then

   -- Don't get notification for other addons being loaded.
   this:UnregisterEvent("ADDON_LOADED");

   -- Initialize the options.
   MikSBTOpt.Init();
  end
 end
end


-- **********************************************************************************
-- Called when the mod is fully loaded (variables loaded).
-- **********************************************************************************
function MikSBTOpt.Init()
 -- Set the options window title.
 getglobal(OPTIONS_FRAME_NAME .. "TitleText"):SetText(MikSBT.WINDOW_TITLE);

 -- Set the normal and crit size font preview text.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "NormalPreviewText"):SetText(MikSBTOpt.MSG_NORMAL_PREVIEW_TEXT);
 getglobal(FONT_SETTINGS_FRAME_NAME .. "CritPreviewText"):SetText(MikSBTOpt.MSG_CRIT_PREVIEW_TEXT);

 -- Setup the the tabs.
 MikSBTOpt.SetupTabs();

 -- Setup the dropdowns.
 MikSBTOpt.SetupDropdowns();

 -- Setup the checkboxes.
 MikSBTOpt.SetupCheckboxes();

 -- Setup the sliders.
 MikSBTOpt.SetupSliders();

 -- Setup the buttons.
 MikSBTOpt.SetupButtons();

 -- Setup the editboxes.
 MikSBTOpt.SetupEditboxes();

 -- Setup the color swatch interface data.
 MikSBTOpt.SetupInterfaceColorSwatchData();

 -- Setup the interface event data.
 MikSBTOpt.SetupInterfaceEventData();


 -- Populates the tabs and selects the first one.
 MikSBTOpt.PopulateTabs();
 MikSBTOpt.TabOnClick(1);


 -- Populate the Disable MSBT checkbox with the correct value.
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_DISABLE_MSBT, MikSBT_Save.UserDisabled);


 -- Populate the profile dropdown with the current profile, change the state of the profile buttons correctly, and set the current profile label.
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_PROFILE, MikSBT_Save.CurrentProfile);
 MikSBTOpt.EnableDisableProfileButtons();
 getglobal(OPTIONS_FRAME_NAME .. "Tab1FrameCurrentProfileLabel"):SetText(MikSBTOpt.MSG_CURRENT_PROFILE_TEXT .. ": " .. MikSBT_Save.CurrentProfile);


 -- Populate the options for the current profile.
 MikSBTOpt.PopulateProfileOptions();

 -- Insert the frame name into the UISpecialFrames array so it closes when the escape key is pressed.
 table.insert(UISpecialFrames, OPTIONS_FRAME_NAME);
end


-- **********************************************************************************
-- Called when the options frame is shown.
-- **********************************************************************************
function MikSBTOpt.OnShow()
 -- Holds whether or not to play a sound.
 local doPlaySound = true;

 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Check if the scroll area mover frame is shown.
  if (frame:IsShown()) then
   -- Don't play a sound.
   doPlaySound = false;
  end
 end

 -- Check if a sound should be played.
 if (doPlaySound) then
  -- Play the opening sound
  PlaySound("igCharacterInfoOpen");
 end
end


-- **********************************************************************************
-- Called when the options frame is hidden.
-- **********************************************************************************
function MikSBTOpt.OnHide()
 -- Holds whether or not to play a sound.
 local doPlaySound = true;

 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Check if the scroll area mover frame is shown.
  if (frame:IsShown()) then
   -- Don't play a sound.
   doPlaySound = false;
  end
 end

 -- Check if a sound should be played.
 if (doPlaySound) then
  -- Play the closing sound
  PlaySound("igCharacterInfoClose");
 end

 -- Hide the font settings frame when the main frame is closed.
 getglobal(FONT_SETTINGS_FRAME_NAME):Hide();
end


-- **********************************************************************************
-- Called when one of the event settings areas is scrolled.
-- **********************************************************************************
function MikSBTOpt.OnVerticalScroll()
 -- Check if it's the incoming event settings scroll frame.
 if (this:GetName() == OPTIONS_FRAME_NAME .. "Tab2FrameEventsScrollFrame") then
  FauxScrollFrame_OnVerticalScroll(EVENT_ITEM_HEIGHT, MikSBTOpt.PopulateIncomingEventSettings);

 -- Outgoing event settings scroll frame.
 elseif (this:GetName() == OPTIONS_FRAME_NAME .. "Tab3FrameEventsScrollFrame") then
  FauxScrollFrame_OnVerticalScroll(EVENT_ITEM_HEIGHT, MikSBTOpt.PopulateOutgoingEventSettings);

 -- Notification event settings scroll frame.
 elseif (this:GetName() == OPTIONS_FRAME_NAME .. "Tab4FrameEventsScrollFrame") then
  FauxScrollFrame_OnVerticalScroll(EVENT_ITEM_HEIGHT, MikSBTOpt.PopulateNotificationEventSettings);

 end
end


-------------------------------------------------------------------------------------
-- Interface data setup functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up the interface tab data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceTabData()
 local index;
 local objectData;
 interfaceData.Tabs = MikSBTOpt.TABS;

 local ifData = interfaceData.Tabs;
 index = 1; objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1";
 index = 2; objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2";
 index = 3; objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3";
 index = 4; objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4";
end


-- **********************************************************************************
-- Sets up the interface dropdown data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceDropdownData()
 -- Setup the available fonts with the inherit option.
 availableFontsInherit = MikSBT.CopyTable(MikSBT.AVAILABLE_FONTS);
 table.insert(availableFontsInherit, 1, {Name=MikSBTOpt.MSG_INHERIT_TEXT});
 for key, data in availableFontsInherit do
  data.Value = key - 1;
 end

 -- Setup the available font outlines with the inherit option.
 availableOutlinesInherit = MikSBT.CopyTable(MikSBT.AVAILABLE_OUTLINES);
 table.insert(availableOutlinesInherit, 1, {Name=MikSBTOpt.MSG_INHERIT_TEXT});
 for key, data in availableOutlinesInherit do
  data.Value = key - 1;
 end

 -- Setup the available profiles.
 local availableProfiles = {};
 for profileName, _ in MikSBT_Save.Profiles do
  availableProfiles[profileName] = {Name=profileName};
 end


 local index;
 local objectData;
 interfaceData.Dropdowns = MikSBTOpt.DROPDOWNS;

 local ifData = interfaceData.Dropdowns;
 index = DROPDOWN_INDEX_PROFILE;					objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameProfileDropdown";		objectData.VarPath=nil;											objectData.ClickFunction=MikSBTOpt.ProfileDropdownOnClick;		objectData.Options=availableProfiles;
 index = DROPDOWN_INDEX_INCOMING_TEXT_ALIGN;			objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2FrameTextAlignDropdown";		objectData.VarPath="DisplaySettings.Incoming.FontSettings.TextAlignIndex";		objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_TEXT_ALIGNS;
 index = DROPDOWN_INDEX_INCOMING_ANIMATION_STYLE;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2FrameAnimationStyleDropdown";	objectData.VarPath="DisplaySettings.Incoming.AnimationStyle";				objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_ANIMATION_STYLES;
 index = DROPDOWN_INDEX_INCOMING_SCROLL_DIRECTION;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2FrameScrollDirectionDropdown";	objectData.VarPath="DisplaySettings.Incoming.ScrollDirection";				objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_SCROLL_DIRECTIONS;
 index = DROPDOWN_INDEX_OUTGOING_TEXT_ALIGN;			objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3FrameTextAlignDropdown";		objectData.VarPath="DisplaySettings.Outgoing.FontSettings.TextAlignIndex";		objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_TEXT_ALIGNS;
 index = DROPDOWN_INDEX_OUTGOING_ANIMATION_STYLE;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3FrameAnimationStyleDropdown";	objectData.VarPath="DisplaySettings.Outgoing.AnimationStyle";				objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_ANIMATION_STYLES;
 index = DROPDOWN_INDEX_OUTGOING_SCROLL_DIRECTION;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3FrameScrollDirectionDropdown";	objectData.VarPath="DisplaySettings.Outgoing.ScrollDirection";				objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_SCROLL_DIRECTIONS;
 index = DROPDOWN_INDEX_NOTIFICATION_TEXT_ALIGN;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4FrameTextAlignDropdown";		objectData.VarPath="DisplaySettings.Notification.FontSettings.TextAlignIndex";	objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_TEXT_ALIGNS;
 index = DROPDOWN_INDEX_NOTIFICATION_ANIMATION_STYLE;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4FrameAnimationStyleDropdown";	objectData.VarPath="DisplaySettings.Notification.AnimationStyle";				objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_ANIMATION_STYLES;
 index = DROPDOWN_INDEX_NOTIFICATION_SCROLL_DIRECTION;	objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4FrameScrollDirectionDropdown";	objectData.VarPath="DisplaySettings.Notification.ScrollDirection";			objectData.ClickFunction=MikSBTOpt.DropdownOnClick;			objectData.Options=MikSBT.AVAILABLE_SCROLL_DIRECTIONS;

 index = DROPDOWN_INDEX_FONT;						objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "FontDropdown";			objectData.ClickFunction=MikSBTOpt.FontSettingsDropdownOnClick;
 index = DROPDOWN_INDEX_FONT_OUTLINE;				objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "FontOutlineDropdown";		objectData.ClickFunction=MikSBTOpt.FontSettingsDropdownOnClick;
 index = DROPDOWN_INDEX_FONT_CRIT;					objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "FontCritDropdown";		objectData.ClickFunction=MikSBTOpt.FontSettingsDropdownOnClick;
 index = DROPDOWN_INDEX_FONT_OUTLINE_CRIT;			objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "FontOutlineCritDropdown";	objectData.ClickFunction=MikSBTOpt.FontSettingsDropdownOnClick;
end


-- **********************************************************************************
-- Sets up the interface checkbox data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceCheckboxData()
 local index;
 local objectData;
 interfaceData.Checkboxes = MikSBTOpt.CHECKBOXES;

 local ifData = interfaceData.Checkboxes;
 index = CHECKBOX_INDEX_DISABLE_MSBT;			objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameDisableCheckbox";			objectData.VarPath=nil;
 index = CHECKBOX_INDEX_SHOW_PARTIAL_EFFECTS;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameShowPartialEffectsCheckbox";	objectData.VarPath="ShowPartialEffects";
 index = CHECKBOX_INDEX_USE_STICKY_CRITS;			objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameUseStickyCritsCheckbox";		objectData.VarPath="UseStickyCrits";
 index = CHECKBOX_INDEX_SHOW_OVERHEALS;			objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameShowOverhealsCheckbox";		objectData.VarPath="ShowOverheals";
 index = CHECKBOX_INDEX_SHOW_GAME_DAMAGE;			objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameShowGameDamageCheckbox";		objectData.VarPath="ShowGameDamage";
 index = CHECKBOX_INDEX_SHOW_INCOMING_EVENTS;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2FrameShowEventsCheckbox";			objectData.VarPath="DisplaySettings.Incoming.Show";
 index = CHECKBOX_INDEX_SHOW_OUTGOING_EVENTS;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3FrameShowEventsCheckbox";			objectData.VarPath="DisplaySettings.Outgoing.Show";
 index = CHECKBOX_INDEX_SHOW_NOTIFICATION_EVENTS;	objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4FrameShowEventsCheckbox";			objectData.VarPath="DisplaySettings.Notification.Show";
 index = CHECKBOX_INDEX_INHERIT_FONT_SIZE;		objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox";
 index = CHECKBOX_INDEX_INHERIT_FONT_SIZE_CRIT;		objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox";
end


-- **********************************************************************************
-- Sets up the interface slider data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceSliderData()
 local index;
 local objectData;
 interfaceData.Sliders = MikSBTOpt.SLIDERS;

 local ifData = interfaceData.Sliders;
 index = SLIDER_INDEX_INCOMING_SCROLL_HEIGHT;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2FrameScrollHeightSlider";	objectData.VarPath="DisplaySettings.Incoming.ScrollHeight";					objectData.MinValue=MIN_SCROLL_HEIGHT;	objectData.MaxValue=MAX_SCROLL_HEIGHT;	objectData.ValueStep=2;
 index = SLIDER_INDEX_OUTGOING_SCROLL_HEIGHT;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3FrameScrollHeightSlider";	objectData.VarPath="DisplaySettings.Outgoing.ScrollHeight";					objectData.MinValue=MIN_SCROLL_HEIGHT;	objectData.MaxValue=MAX_SCROLL_HEIGHT;	objectData.ValueStep=2;
 index = SLIDER_INDEX_NOTIFICATION_SCROLL_HEIGHT;	objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4FrameScrollHeightSlider";	objectData.VarPath="DisplaySettings.Notification.ScrollHeight";				objectData.MinValue=MIN_SCROLL_HEIGHT;	objectData.MaxValue=MAX_SCROLL_HEIGHT;	objectData.ValueStep=2;
 index = SLIDER_INDEX_ANIMATION_STEP;			objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameAnimationStepSlider";	objectData.VarPath="AnimationStep";									objectData.MinValue=MIN_ANIMATION_STEP;		objectData.MaxValue=MAX_ANIMATION_STEP;		objectData.ValueStep=1;

 index = SLIDER_INDEX_FONT_SIZE;				objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider";			objectData.MinValue=MIN_FONT_SIZE;	objectData.MaxValue=MAX_FONT_SIZE;	objectData.ValueStep=2;
 index = SLIDER_INDEX_FONT_SIZE_CRIT;			objectData = ifData[index]; objectData.FrameName=FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider";		objectData.MinValue=MIN_FONT_SIZE;	objectData.MaxValue=MAX_FONT_SIZE;	objectData.ValueStep=2;
end

-- **********************************************************************************
-- Sets up the interface button data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceButtonData()
 local index;
 local objectData;
 interfaceData.Buttons = MikSBTOpt.BUTTONS;

 local ifData = interfaceData.Buttons;
 index = BUTTON_INDEX_EDIT_MASTER_FONT_SETTINGS;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameEditFontSettingsButton";
 index = BUTTON_INDEX_EDIT_INCOMING_FONT_SETTINGS;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2FrameEditFontSettingsButton";
 index = BUTTON_INDEX_EDIT_OUTGOING_FONT_SETTINGS;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3FrameEditFontSettingsButton";
 index = BUTTON_INDEX_EDIT_NOTIFICATION_FONT_SETTINGS;	objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4FrameEditFontSettingsButton";
 index = BUTTON_INDEX_LOAD_PROFILE;					objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameLoadProfileButton";
 index = BUTTON_INDEX_DELETE_PROFILE;				objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameDeleteProfileButton";
 index = BUTTON_INDEX_CREATE_PROFILE;				objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameCreateProfileButton";
 index = BUTTON_INDEX_RESET_PROFILE;				objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameResetProfileButton";
 index = BUTTON_INDEX_MOVE_INCOMING_SCROLL_AREA;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab2FrameMoveScrollAreaButton";
 index = BUTTON_INDEX_MOVE_OUTGOING_SCROLL_AREA;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab3FrameMoveScrollAreaButton";
 index = BUTTON_INDEX_MOVE_NOTIFICATION_SCROLL_AREA;		objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab4FrameMoveScrollAreaButton";
 index = BUTTON_INDEX_SCROLL_AREA_MOVER_PREVIEW;		objectData = ifData[index]; objectData.FrameName=SCROLL_AREA_MOVER_FRAME_NAME .. "PreviewButton";
 index = BUTTON_INDEX_SCROLL_AREA_MOVER_SAVE;			objectData = ifData[index]; objectData.FrameName=SCROLL_AREA_MOVER_FRAME_NAME .. "SaveButton";
 index = BUTTON_INDEX_SCROLL_AREA_MOVER_CANCEL;			objectData = ifData[index]; objectData.FrameName=SCROLL_AREA_MOVER_FRAME_NAME .. "CancelButton";
end

-- **********************************************************************************
-- Sets up the interface edit box data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceEditboxData()
 local index;
 local objectData;
 interfaceData.Editboxes = MikSBTOpt.EDITBOXES;

 local ifData = interfaceData.Editboxes;
 index = EDITBOX_INDEX_NEW_PROFILE_NAME;	objectData = ifData[index]; objectData.FrameName=OPTIONS_FRAME_NAME .. "Tab1FrameNewProfileNameEditbox";
end

-- **********************************************************************************
-- Sets up the interface color swatch data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceColorSwatchData()
 interfaceData.ColorSwatches = MikSBTOpt.COLORSWATCHES;
end

-- **********************************************************************************
-- Sets up the interface event data.
-- **********************************************************************************
function MikSBTOpt.SetupInterfaceEventData()
 local index;
 local objectData;

 interfaceData.IncomingEvents = MikSBTOpt.INCOMING_EVENTS;
 local ifData = interfaceData.IncomingEvents;
 index = 1;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_DAMAGE";
 index = 2;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_MISS";
 index = 3;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_DODGE";
 index = 4;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_PARRY";
 index = 5;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_BLOCK";
 index = 6;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_ABSORB";
 index = 7;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_IMMUNE";
 index = 8;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_DAMAGE";
 index = 9;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_DOT";
 index = 10;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_MISS";
 index = 11;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_DODGE";
 index = 12;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_PARRY";
 index = 13;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_BLOCK";
 index = 14;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_RESIST";
 index = 15;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_ABSORB";
 index = 16;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_IMMUNE";
 index = 17;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_SPELL_REFLECT";
 index = 18;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_HEAL";
 index = 19;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_HOT";
 index = 20;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_INCOMING_ENVIRONMENTAL";


 interfaceData.OutgoingEvents = MikSBTOpt.OUTGOING_EVENTS;
 local ifData = interfaceData.OutgoingEvents;
 index = 1;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_DAMAGE";
 index = 2;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_MISS";
 index = 3;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_DODGE";
 index = 4;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PARRY";
 index = 5;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_BLOCK";
 index = 6;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_ABSORB";
 index = 7;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_IMMUNE";
 index = 8;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_EVADE";
 index = 9;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_DAMAGE";
 index = 10;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_DOT";
 index = 11;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_MISS";
 index = 12;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_DODGE";
 index = 13;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_PARRY";
 index = 14;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_BLOCK";
 index = 15;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_RESIST";
 index = 16;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_ABSORB";
 index = 17;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_IMMUNE";
 index = 18;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_REFLECT";
 index = 19;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_SPELL_EVADE";
 index = 20;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_HEAL";
 index = 21;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_HOT";
 index = 22;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_DAMAGE";
 index = 23;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_MISS";
 index = 24;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_DODGE";
 index = 25;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_PARRY";
 index = 26;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_BLOCK";
 index = 27;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_ABSORB";
 index = 28;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_IMMUNE";
 index = 29;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_EVADE";
 index = 30;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_DAMAGE";
 index = 31;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_MISS";
 index = 32;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_DODGE";
 index = 33;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_PARRY";
 index = 34;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_BLOCK";
 index = 35;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_RESIST";
 index = 36;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_ABSORB";
 index = 37;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_IMMUNE";
 index = 38;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_OUTGOING_PET_SPELL_EVADE";


 interfaceData.NotificationEvents = MikSBTOpt.NOTIFICATION_EVENTS;
 local ifData = interfaceData.NotificationEvents;
 index = 1;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_DEBUFF";
 index = 2;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_BUFF";
 index = 3;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_ITEM_BUFF";
 index = 4;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_BUFF_FADE";
 index = 5;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_COMBAT_ENTER";
 index = 6;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_COMBAT_LEAVE";
 index = 7;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_POWER_GAIN";
 index = 8;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_POWER_LOSS";
 index = 9;		objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_CP_GAIN";
 index = 10;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_CP_FULL";
 index = 11;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_HONOR_GAIN";
 index = 12;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_REP_GAIN";
 index = 13;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_REP_LOSS";
 index = 14;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_SKILL_GAIN";
 index = 15;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_EXPERIENCE_GAIN";
 index = 16;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_LOW_HEALTH";
 index = 17;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_LOW_MANA";
 index = 18;	objectData = ifData[index]; objectData.EventType = "MSBT_EVENTTYPE_NOTIFICATION_EXECUTE";
end


-------------------------------------------------------------------------------------
-- Tab functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up the tabs.
-- **********************************************************************************
function MikSBTOpt.SetupTabs()
 -- Sets up the interface tab data.
 MikSBTOpt.SetupInterfaceTabData();

 -- Set the number of tabs for the options frame.
 PanelTemplates_SetNumTabs(getglobal(OPTIONS_FRAME_NAME), table.getn(interfaceData.Tabs));
end


-- **********************************************************************************
-- Populates the tabs and resizes them to fit it.
-- **********************************************************************************
function MikSBTOpt.PopulateTabs()
 -- Loop through all of the tabs.
 for _, tabData in interfaceData.Tabs do
  -- Get the associated tab frame.
  local tabFrame = getglobal(tabData.FrameName);
  if (tabFrame) then
   -- Set the tooltip for the tab.
   tabFrame.tooltipText = tabData.Tooltip;

   -- Set the text for the tab and resize it.
   tabFrame:SetText(tabData.Label);
   PanelTemplates_TabResize(1, tabFrame);
  end  
 end
end

-- **********************************************************************************
-- Called when a tab is clicked.
-- **********************************************************************************
function MikSBTOpt.TabOnClick(tab)
 -- Get the clicked tab
 if (not tab) then
  tab = this:GetID();
 end

 -- Set the selected tab
 PanelTemplates_SetTab(getglobal(OPTIONS_FRAME_NAME), tab);

 -- Hide all of the tab frames.
 for _, tabData in interfaceData.Tabs do
  local associatedTabFrame = getglobal(tabData.FrameName .. "Frame");
  if (associatedTabFrame) then
   associatedTabFrame:Hide();
  end
 end

 -- Show the associated frame for the selected tab.
 local associatedTabFrame = getglobal(interfaceData.Tabs[tab].FrameName .. "Frame");
 if (associatedTabFrame) then
  associatedTabFrame:Show();
 end

 -- Hide the font settings frame.
 getglobal(FONT_SETTINGS_FRAME_NAME):Hide();
end


-------------------------------------------------------------------------------------
-- Dropdown functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up all the dropdowns.
-- **********************************************************************************
function MikSBTOpt.SetupDropdowns()
 -- Setup the dropdown interface data.
 MikSBTOpt.SetupInterfaceDropdownData();

 -- Setup the profile dropdown.
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_PROFILE);

 -- Setup the text align dropdowns
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_INCOMING_TEXT_ALIGN);
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_OUTGOING_TEXT_ALIGN);
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_NOTIFICATION_TEXT_ALIGN);

 -- Setup the animation styles dropdowns.
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_INCOMING_ANIMATION_STYLE);
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_OUTGOING_ANIMATION_STYLE);
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_NOTIFICATION_ANIMATION_STYLE);

 -- Setup the scroll direction dropdowns.
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_INCOMING_SCROLL_DIRECTION);
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_OUTGOING_SCROLL_DIRECTION);
 MikSBTOpt.SetupDropdown(DROPDOWN_INDEX_NOTIFICATION_SCROLL_DIRECTION);
end


-- **********************************************************************************
-- Sets up the dropdown for the passed data index.
-- **********************************************************************************
function MikSBTOpt.SetupDropdown(dropdownDataIndex)
 local dropdownData = interfaceData.Dropdowns[dropdownDataIndex];

 -- Make sure there is data for the dropdown.
 if (dropdownData) then
  local associatedFrame = getglobal(dropdownData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   associatedFrame.dropdownDataIndex = dropdownDataIndex;
   associatedFrame.associatedVarPath = dropdownData.VarPath;
   associatedFrame.tooltipText = dropdownData.Tooltip;

   -- Set the label for the dropdown.
   getglobal(dropdownData.FrameName .. "Label"):SetText(dropdownData.Label);

   -- Initialize the dropdown.
   UIDropDownMenu_Initialize(associatedFrame, MikSBTOpt.DropdownInit);

   -- Setup the default value.
   MikSBTOpt.PopulateDropdown(dropdownDataIndex);
  end
 end
end


-- **********************************************************************************
-- Sets the dropdown's selected value for the passed data index.  If a value is passed
-- it is used, otherwise the function will attempt to get the selected value from
-- the VarPath field of the dropdown data.
-- **********************************************************************************
function MikSBTOpt.PopulateDropdown(dropdownDataIndex, selectValue)
 local dropdownData = interfaceData.Dropdowns[dropdownDataIndex];

 -- Make sure there is data for the dropdown.
 if (dropdownData) then
  local associatedFrame = getglobal(dropdownData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   local tempValue = selectValue;

   -- Check if no selected value was passed.
   if (tempValue == nil) then
    -- Check if there is an associated variable path for the dropdown.
    if (dropdownData.VarPath) then
     -- Get the selected value from the current profile.
     tempValue = MikSBT.GetOptionFromVarPath(dropdownData.VarPath);
    end
   end

   -- Make sure there is a value to be selected.
   if (tempValue ~= nil) then
    -- Set the default selected value.
    UIDropDownMenu_SetSelectedValue(associatedFrame, tempValue);

    -- Set the text for the default selected value.
    for key, optionData in dropdownData.Options do
     -- If there is no value for the option data, use the key as the value.
     local optionValue = optionData.Value;
     if (optionValue == nil) then
      optionValue = key;
     end
     local selectedValue = associatedFrame.selectedValue;

     if (optionValue == selectedValue) then
      UIDropDownMenu_SetText(optionData.Name, associatedFrame);
     end
    end
   end
  end
 end 
end


-- **********************************************************************************
-- Called when a drop down menu is initialized (shown).
-- **********************************************************************************
function MikSBTOpt.DropdownInit()
 local info = {};

 -- Get the dropdown data index.
 local dropdownDataIndex = this.dropdownDataIndex;
 if (dropdownDataIndex == nil) then
  dropdownDataIndex = this:GetParent().dropdownDataIndex;
 end

 -- Get the dropdown data.
 local dropdownData = interfaceData.Dropdowns[dropdownDataIndex];

 -- Make sure there is dropdown data.
 if (dropdownData) then
  -- Loop through all of the options for dropdown.
  for key, optionData in dropdownData.Options do

   -- If the option data has a value use it otherwise use the key as the value.
   local tempValue = optionData.Value;
   if (tempValue == nil) then
    tempValue = key;
   end

   -- Add the option to the dropdown list.
   info = {};
   info.text = optionData.Name;
   info.value = tempValue;
   info.func = dropdownData.ClickFunction;
   info.arg1 = dropdownData.FrameName;
   UIDropDownMenu_AddButton(info);
  end
 end
end


-- **********************************************************************************
-- Called when one of the dropdown menu options is clicked.
-- **********************************************************************************
function MikSBTOpt.DropdownOnClick(frameName)
 -- Set the selected dropdown option to the appropriate value.
 UIDropDownMenu_SetSelectedValue(getglobal(frameName), this.value);

 -- Get the associated variable path.
 local associatedVarPath = getglobal(frameName).associatedVarPath;

 -- Make sure there is a variable path.
 if (associatedVarPath) then
  -- Update current profile data.
  MikSBT.SetOptionFromVarPath(associatedVarPath, this.value);
 end
end


-- **********************************************************************************
-- Called when one of the dropdown menu options on the font settings frame is clicked.
-- **********************************************************************************
function MikSBTOpt.FontSettingsDropdownOnClick(frameName)
 -- Call the normal dropdown OnClick handler.
 MikSBTOpt.DropdownOnClick(frameName);
 
 -- Update the font previews.
 MikSBTOpt.UpdateFontPreviews(); 
end


-- **********************************************************************************
-- Called when one of the profile dropdown menu options is clicked.
-- **********************************************************************************
function MikSBTOpt.ProfileDropdownOnClick(frameName)
 -- Call the normal dropdown OnClick handler.
 MikSBTOpt.DropdownOnClick(frameName);

 -- Enable and disable the load and delete profile buttons.
 MikSBTOpt.EnableDisableProfileButtons();
end

-- This function refreshes the options for the profiles dropdown.
function MikSBTOpt.RefreshProfilesDropdown()
 -- Setup the available profiles.
 local availableProfiles = {};
 for profileName, _ in MikSBT_Save.Profiles do
  availableProfiles[profileName] = {Name=profileName};
 end

 -- Reset the available options for the profiles dropdown.
 interfaceData.Dropdowns[DROPDOWN_INDEX_PROFILE].Options = availableProfiles;
end



-------------------------------------------------------------------------------------
-- Checkbox functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up all the checkboxes.
-- **********************************************************************************
function MikSBTOpt.SetupCheckboxes()
 -- Setup the checkbox interface data.
 MikSBTOpt.SetupInterfaceCheckboxData();

 -- Setup the Disable MSBT checkbox.
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_DISABLE_MSBT);

 -- Setup the Show Partial Effects checkbox.
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_SHOW_PARTIAL_EFFECTS);

 -- Setup the Use Sticky Crits checkbox.
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_USE_STICKY_CRITS);

 -- Setup the Use Sticky Crits checkbox.
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_SHOW_OVERHEALS);

 -- Setup the Use Sticky Crits checkbox.
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_SHOW_GAME_DAMAGE);



 -- Setup the show scroll area events checkboxes.
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_SHOW_INCOMING_EVENTS);
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_SHOW_OUTGOING_EVENTS);
 MikSBTOpt.SetupCheckbox(CHECKBOX_INDEX_SHOW_NOTIFICATION_EVENTS); 
end


-- **********************************************************************************
-- Sets up the checkbox for passed data index.
-- **********************************************************************************
function MikSBTOpt.SetupCheckbox(checkboxDataIndex)
 local checkboxData = interfaceData.Checkboxes[checkboxDataIndex];

 -- Make sure there is data for the checkbox.
 if (checkboxData) then
  local associatedFrame = getglobal(checkboxData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   associatedFrame.checkboxDataIndex = checkboxDataIndex;
   associatedFrame.associatedVarPath = checkboxData.VarPath;
   associatedFrame.tooltipText = checkboxData.Tooltip;

   -- Set the text for the checkbox.
   if (checkboxData.Label) then
    getglobal(checkboxData.FrameName .. "Text"):SetText(checkboxData.Label);
   end

   -- Set the default value.
   MikSBTOpt.PopulateCheckbox(checkboxDataIndex);
  end
 end
end

-- **********************************************************************************
-- Sets the checkbox's checked state for the passed data index.  If a value is passed
-- it is used, otherwise the function will attempt to get the selected value from
-- the VarPath field of the checkbox data.
-- **********************************************************************************
function MikSBTOpt.PopulateCheckbox(checkboxDataIndex, isChecked)
 local checkboxData = interfaceData.Checkboxes[checkboxDataIndex];

 -- Make sure there is data for the checkbox.
 if (checkboxData) then
  local associatedFrame = getglobal(checkboxData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   local tempValue = isChecked;

   -- Check if no value was passed.
   if (tempValue == nil) then
    -- Check if there is an associated variable path for the checkbox.
    if (checkboxData.VarPath) then
     -- Get the selected value from the current profile.
     tempValue = MikSBT.GetOptionFromVarPath(checkboxData.VarPath);
    end
   end

   -- Make sure there is a value to set.
   if (tempValue ~= nil) then
    -- Set the checkbox checked state appropriately.
    if (tempValue == true) then
     associatedFrame:SetChecked(1);
    else
     associatedFrame:SetChecked(0);
    end     
   end
  end
 end 
end


-- **********************************************************************************
-- Called when a checkbox is clicked.
-- **********************************************************************************
function MikSBTOpt.CheckboxOnClick()
 -- Disable MSBT checkbox.
 if (this.checkboxDataIndex == CHECKBOX_INDEX_DISABLE_MSBT) then
  -- Check if the checkbox is checked.
  if (this:GetChecked() == 1) then
   MikSBT.SetOptionUserDisabled(true);
  else
   MikSBT.SetOptionUserDisabled(false);
  end

  -- Return from the function.
  return;

 elseif (this.checkboxDataIndex == CHECKBOX_INDEX_SHOW_GAME_DAMAGE) then
  -- Check if the checkbox is checked.
  if (this:GetChecked() == 1) then
   MikSBT.SetOptionShowGameDamage(true);
  else
   MikSBT.SetOptionShowGameDamage(false);
  end

  -- Return from the function.
  return;

 -- Inherit font size, or inherit critical font size checkbox.
 elseif (this.checkboxDataIndex == CHECKBOX_INDEX_INHERIT_FONT_SIZE or
         this.checkboxDataIndex == CHECKBOX_INDEX_INHERIT_FONT_SIZE_CRIT) then

  -- Call the font settings checkbox OnClick handler.
  MikSBTOpt.FontSettingsCheckboxOnClick();

  -- Return from the function.
  return;

 end

 -- Check if there is an associated variable path and set it to the correct value.
 if (this.associatedVarPath) then
  if (this:GetChecked() == 1) then
   -- Set the associated variable in the current profile to the inherited value.
   MikSBT.SetOptionFromVarPath(this.associatedVarPath, true);
  else
   -- Set the associated variable in the current profile to the inherited value.
   MikSBT.SetOptionFromVarPath(this.associatedVarPath, false);
  end
 end
end


-- **********************************************************************************
-- Called when a checkbox in the font settings area is clicked.
-- **********************************************************************************
function MikSBTOpt.FontSettingsCheckboxOnClick()
 local associatedSlider;
 local inheritedValue;
 local sliderDataIndex;

 -- Check if it's the inherit font size checkbox.
 if (this.checkboxDataIndex == CHECKBOX_INDEX_INHERIT_FONT_SIZE) then
  associatedSlider = getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider");
  inheritedValue = this:GetParent().InheritedFontSize;
  sliderDataIndex = SLIDER_INDEX_FONT_SIZE;
 elseif (this.checkboxDataIndex == CHECKBOX_INDEX_INHERIT_FONT_SIZE_CRIT) then
  associatedSlider = getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider");
  inheritedValue = this:GetParent().InheritedFontSizeCrit;
  sliderDataIndex = SLIDER_INDEX_FONT_SIZE_CRIT;
 end
  
 -- Make sure there is an associated variable path.
 if (this.associatedVarPath) then
  if (this:GetChecked() == 1) then
   -- Set the associated variable in the current profile to inherit.
   MikSBT.SetOptionFromVarPath(this.associatedVarPath, 0);

   -- Hide the associated slider.
   associatedSlider:Hide();
  else
   -- Set the associated variable in the current profile to the inherited value.
   MikSBT.SetOptionFromVarPath(this.associatedVarPath, inheritedValue);

   -- Show the associated slider.
   associatedSlider:Show();

   -- Setup the slider.
   MikSBTOpt.SetupSlider(sliderDataIndex);
  end
 end

 -- Update the font previews.
 MikSBTOpt.UpdateFontPreviews();
end


-------------------------------------------------------------------------------------
-- Slider functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up all the sliders.
-- **********************************************************************************
function MikSBTOpt.SetupSliders()
 -- Setup the slider interface data.
 MikSBTOpt.SetupInterfaceSliderData();

 -- Setup Animation Step slider.
 MikSBTOpt.SetupSlider(SLIDER_INDEX_ANIMATION_STEP);

 -- Setup the scroll height sliders.
 MikSBTOpt.SetupSlider(SLIDER_INDEX_INCOMING_SCROLL_HEIGHT);
 MikSBTOpt.SetupSlider(SLIDER_INDEX_OUTGOING_SCROLL_HEIGHT);
 MikSBTOpt.SetupSlider(SLIDER_INDEX_NOTIFICATION_SCROLL_HEIGHT);
end


-- **********************************************************************************
-- Sets up the slider for the passed data index.
-- **********************************************************************************
function MikSBTOpt.SetupSlider(sliderDataIndex)
 local sliderData = interfaceData.Sliders[sliderDataIndex];

 -- Make sure there is data for the slider.
 if (sliderData) then
  local associatedSliderFrame = getglobal(sliderData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedSliderFrame) then
   associatedSliderFrame.sliderDataIndex = sliderDataIndex;
   associatedSliderFrame.associatedVarPath = sliderData.VarPath;
   associatedSliderFrame.tooltipText = sliderData.Tooltip;

   -- Set the text for the slider.
   getglobal(sliderData.FrameName .. "Text"):SetText(sliderData.Label);

   -- Set the min/max values for the slider.
   associatedSliderFrame:SetMinMaxValues(sliderData.MinValue, sliderData.MaxValue);
   associatedSliderFrame:SetValueStep(sliderData.ValueStep);

   -- Sets the default value.
   MikSBTOpt.PopulateSlider(sliderDataIndex);
  end
 end
end

-- **********************************************************************************
-- Sets the slider's value for the passed data index.  If a value is passed
-- it is used, otherwise the function will attempt to get the selected value from
-- the VarPath field of the slider data.
-- **********************************************************************************
function MikSBTOpt.PopulateSlider(sliderDataIndex, sliderValue)
 local sliderData = interfaceData.Sliders[sliderDataIndex];

 -- Make sure there is data for the slider.
 if (sliderData) then
  local associatedSliderFrame = getglobal(sliderData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedSliderFrame) then
   local tempValue = sliderValue;

   -- Check if no value was passed.
   if (tempValue == nil) then
    -- Check if there is an associated variable path for the slider.
    if (sliderData.VarPath) then
     -- Get the selected value from the current profile.
     tempValue = MikSBT.GetOptionFromVarPath(sliderData.VarPath);
    end
   end

   -- Make sure there is a value to set.
   if (tempValue ~= nil) then
    -- Set the slider's value to the appropriate value.
    associatedSliderFrame:SetValue(tempValue);

    -- Update the label to reflect the new value.
    getglobal(sliderData.FrameName .. "Text"):SetText(sliderData.Label .. ": " .. associatedSliderFrame:GetValue());
   end

  end
 end
end

-- **********************************************************************************
-- Called when a slider values changes.
-- **********************************************************************************
function MikSBTOpt.SliderOnValueChanged()
 local sliderData = interfaceData.Sliders[this.sliderDataIndex];

 -- Make sure there is data for the slider.
 if (sliderData) then
  -- Update the label to reflect the new value.
  getglobal(sliderData.FrameName .. "Text"):SetText(sliderData.Label .. ": " .. this:GetValue());
 end

 -- Check if the slider has an associated variable path.
 if (this.associatedVarPath) then
  -- Update current profile data.
  MikSBT.SetOptionFromVarPath(this.associatedVarPath, this:GetValue());
 end
end

-- **********************************************************************************
-- Called when a font settings slider values changes.
-- **********************************************************************************
function MikSBTOpt.FontSettingsSliderOnValueChanged()
 -- Call the normal slider OnValueChanged handler.
 MikSBTOpt.SliderOnValueChanged();

 -- Update the font previews.
 MikSBTOpt.UpdateFontPreviews();
end


-------------------------------------------------------------------------------------
-- Button Functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up all the buttons.
-- **********************************************************************************
function MikSBTOpt.SetupButtons()
 -- Setup the button interface data.
 MikSBTOpt.SetupInterfaceButtonData();

 -- Setup the edit font settings buttons.
 MikSBTOpt.SetupButton(BUTTON_INDEX_EDIT_MASTER_FONT_SETTINGS);
 MikSBTOpt.SetupButton(BUTTON_INDEX_EDIT_INCOMING_FONT_SETTINGS);
 MikSBTOpt.SetupButton(BUTTON_INDEX_EDIT_OUTGOING_FONT_SETTINGS);
 MikSBTOpt.SetupButton(BUTTON_INDEX_EDIT_NOTIFICATION_FONT_SETTINGS);

 -- Setup the profile manipulation buttons.
 MikSBTOpt.SetupButton(BUTTON_INDEX_LOAD_PROFILE);
 MikSBTOpt.SetupButton(BUTTON_INDEX_DELETE_PROFILE);
 MikSBTOpt.SetupButton(BUTTON_INDEX_RESET_PROFILE);
 MikSBTOpt.SetupButton(BUTTON_INDEX_CREATE_PROFILE);

 -- Setup the move scroll area buttons.
 MikSBTOpt.SetupButton(BUTTON_INDEX_MOVE_INCOMING_SCROLL_AREA);
 MikSBTOpt.SetupButton(BUTTON_INDEX_MOVE_OUTGOING_SCROLL_AREA);
 MikSBTOpt.SetupButton(BUTTON_INDEX_MOVE_NOTIFICATION_SCROLL_AREA);

 -- Setup the scroll area mover buttons.
 MikSBTOpt.SetupButton(BUTTON_INDEX_SCROLL_AREA_MOVER_PREVIEW);
 MikSBTOpt.SetupButton(BUTTON_INDEX_SCROLL_AREA_MOVER_SAVE);
 MikSBTOpt.SetupButton(BUTTON_INDEX_SCROLL_AREA_MOVER_CANCEL); 
end

-- **********************************************************************************
-- Sets up the button for the passed data index.
-- **********************************************************************************
function MikSBTOpt.SetupButton(buttonDataIndex)
 local buttonData = interfaceData.Buttons[buttonDataIndex];

 -- Make sure there is data for the button.
 if (buttonData) then
  local associatedFrame = getglobal(buttonData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   associatedFrame.buttonDataIndex = buttonDataIndex;
   associatedFrame.tooltipText = buttonData.Tooltip;

   -- Set the text for the button.
   associatedFrame:SetText(buttonData.Label);
   associatedFrame:SetWidth(associatedFrame:GetTextWidth() + 40);
  end
 end
end


-- **********************************************************************************
-- Called when a button is clicked.
-- **********************************************************************************
function MikSBTOpt.ButtonOnClick()
 -- Edit master font settings button.
 if (this.buttonDataIndex == BUTTON_INDEX_EDIT_MASTER_FONT_SETTINGS) then
  local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);
  fontSettingsFrame:SetPoint("TOPLEFT", this, "BOTTOMLEFT");
  fontSettingsFrame:Raise();
  getglobal(FONT_SETTINGS_FRAME_NAME .. "Title"):SetText(MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Master"].FontSettingsTitle);
  MikSBTOpt.ShowMasterFontSettings();

 -- Edit incoming font settings button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_EDIT_INCOMING_FONT_SETTINGS) then
  -- Hide the event message editbox if it's shown.
  MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab2FrameMessageEditbox");

  -- Get the font settings frame, anchor it to the buton pushed, raise it, 
  -- and change the title.
  local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);
  fontSettingsFrame:SetPoint("TOPLEFT", this, "BOTTOMLEFT");
  fontSettingsFrame:Raise();
  getglobal(FONT_SETTINGS_FRAME_NAME .. "Title"):SetText(MikSBTOpt.TABS[TAB_INDEX_INCOMING].Label);
  MikSBTOpt.ShowScrollAreaFontSettings("Incoming");

 -- Edit outgoing font settings button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_EDIT_OUTGOING_FONT_SETTINGS) then
  -- Hide the event message editbox if it's shown.
  MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab3FrameMessageEditbox");

  -- Get the font settings frame, anchor it to the buton pushed, raise it, 
  -- and change the title.
  local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);
  fontSettingsFrame:SetPoint("TOPLEFT", this, "BOTTOMLEFT");
  fontSettingsFrame:Raise();
  getglobal(FONT_SETTINGS_FRAME_NAME .. "Title"):SetText(MikSBTOpt.TABS[TAB_INDEX_OUTGOING].Label);
  MikSBTOpt.ShowScrollAreaFontSettings("Outgoing");

 -- Edit notification font settings button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_EDIT_NOTIFICATION_FONT_SETTINGS) then
  -- Hide the event message editbox if it's shown.
  MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab4FrameMessageEditbox");

  -- Get the font settings frame, anchor it to the buton pushed, raise it, 
  -- and change the title.
  local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);
  fontSettingsFrame:SetPoint("TOPLEFT", this, "BOTTOMLEFT");
  fontSettingsFrame:Raise();
  getglobal(FONT_SETTINGS_FRAME_NAME .. "Title"):SetText(MikSBTOpt.TABS[TAB_INDEX_NOTIFICATION].Label);
  MikSBTOpt.ShowScrollAreaFontSettings("Notification");

 -- Load profile button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_LOAD_PROFILE) then
  -- Select the appropriate profile and update the current profile label.
  MikSBT.SelectProfile(UIDropDownMenu_GetSelectedValue(getglobal(OPTIONS_FRAME_NAME .. "Tab1FrameProfileDropdown")));
  getglobal(OPTIONS_FRAME_NAME .. "Tab1FrameCurrentProfileLabel"):SetText(MikSBTOpt.MSG_CURRENT_PROFILE_TEXT .. ": " .. MikSBT_Save.CurrentProfile);

  -- Enable and disable the load and delete profile buttons.
  MikSBTOpt.EnableDisableProfileButtons();

  -- Repopulate the options.
  MikSBTOpt.PopulateProfileOptions();

 -- Delete profile button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_DELETE_PROFILE) then
  local profileName = UIDropDownMenu_GetSelectedValue(getglobal(interfaceData.Dropdowns[DROPDOWN_INDEX_PROFILE].FrameName));
  MikSBT.DeleteProfile(profileName);

  -- Refresh the available profiles for the dropdown.
  MikSBTOpt.RefreshProfilesDropdown();

  -- Select the current profile in the dropdown and update the current profile label.
  MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_PROFILE, MikSBT_Save.CurrentProfile);
  getglobal(OPTIONS_FRAME_NAME .. "Tab1FrameCurrentProfileLabel"):SetText(MikSBTOpt.MSG_CURRENT_PROFILE_TEXT .. ": " .. MikSBT_Save.CurrentProfile);

  -- Enable and disable the load and delete profile buttons.
  MikSBTOpt.EnableDisableProfileButtons();

  -- Repopulate the options.
  MikSBTOpt.PopulateProfileOptions();

 -- Reset profile button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_RESET_PROFILE) then
  local profileName = UIDropDownMenu_GetSelectedValue(getglobal(interfaceData.Dropdowns[DROPDOWN_INDEX_PROFILE].FrameName));  
  MikSBT.ResetProfile(profileName, true);

  -- Repopulate the options.
  MikSBTOpt.PopulateProfileOptions();

 -- Create profile button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_CREATE_PROFILE) then
  local profileNameEditbox = getglobal(interfaceData.Editboxes[EDITBOX_INDEX_NEW_PROFILE_NAME].FrameName);
  local profileName = profileNameEditbox:GetText();

  -- Clear the text in the profile name editbox.
  profileNameEditbox:SetText("");

  -- Make sure there is a profile name.
  if (profileName ~= "") then
   MikSBT.CreateProfile(profileName);
   
   -- Refresh the available profiles for the dropdown.
   MikSBTOpt.RefreshProfilesDropdown();

   -- Enable and disable the load and delete profile buttons.
   MikSBTOpt.EnableDisableProfileButtons();

   -- Hide the font settings frame when profile options are populated.
   getglobal(FONT_SETTINGS_FRAME_NAME):Hide();
  end

 -- Move incoming scroll area button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_MOVE_INCOMING_SCROLL_AREA) then
  -- Setup and show the scroll area mover frame.
  MikSBTOpt.ShowIncomingScrollAreaMover();

 -- Move outgoing scroll area button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_MOVE_OUTGOING_SCROLL_AREA) then
  -- Setup and show the scroll area mover frame.
  MikSBTOpt.ShowOutgoingScrollAreaMover();

 -- Move notification scroll area button.
 elseif (this.buttonDataIndex == BUTTON_INDEX_MOVE_NOTIFICATION_SCROLL_AREA) then
  -- Setup and show the scroll area mover frame.
  MikSBTOpt.ShowNotificationScrollAreaMover();

 elseif (this.buttonDataIndex == BUTTON_INDEX_SCROLL_AREA_MOVER_PREVIEW) then
  -- Preview a test message in the new position.
  MikSBTOpt.ScrollAreaMoverPreviewOnClick();

 elseif (this.buttonDataIndex == BUTTON_INDEX_SCROLL_AREA_MOVER_SAVE) then
  -- Save the new position for the scroll area.
  MikSBTOpt.ScrollAreaMoverSaveOnClick();

 elseif (this.buttonDataIndex == BUTTON_INDEX_SCROLL_AREA_MOVER_CANCEL) then
  -- Cancel the scroll area movement.
  MikSBTOpt.ScrollAreaMoverCancelOnClick();

 elseif (this.buttonDataIndex >= BUTTON_INDEX_INCOMING_EVENTS_START and
         this.buttonDataIndex < BUTTON_INDEX_INCOMING_EVENTS_START + EVENT_ITEMS_PER_PAGE) then
  -- Hide the event message editbox if it's shown.
  MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab2FrameMessageEditbox");

  local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);
  fontSettingsFrame:SetPoint("TOPLEFT", this, "BOTTOMLEFT");
  fontSettingsFrame:Raise();

  -- Make sure the event type is set for the button.
  if (this:GetParent().EventType ~= nil) then
   getglobal(FONT_SETTINGS_FRAME_NAME .. "Title"):SetText(this:GetParent().FontFrameTitle);
   MikSBTOpt.ShowEventFontSettings("Incoming", this:GetParent().EventType);
  end

 elseif (this.buttonDataIndex >= BUTTON_INDEX_OUTGOING_EVENTS_START and
         this.buttonDataIndex < BUTTON_INDEX_OUTGOING_EVENTS_START + EVENT_ITEMS_PER_PAGE) then
  -- Hide the event message editbox if it's shown.
  MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab3FrameMessageEditbox");

  local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);
  fontSettingsFrame:SetPoint("TOPLEFT", this, "BOTTOMLEFT");
  fontSettingsFrame:Raise();

  -- Make sure the event type is set for the button.
  if (this:GetParent().EventType ~= nil) then
   getglobal(FONT_SETTINGS_FRAME_NAME .. "Title"):SetText(this:GetParent().FontFrameTitle);
   MikSBTOpt.ShowEventFontSettings("Outgoing", this:GetParent().EventType);
  end

 elseif (this.buttonDataIndex >= BUTTON_INDEX_NOTIFICATION_EVENTS_START and
         this.buttonDataIndex < BUTTON_INDEX_NOTIFICATION_EVENTS_START + EVENT_ITEMS_PER_PAGE) then
  -- Hide the event message editbox if it's shown.
  MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab4FrameMessageEditbox");

  local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);
  fontSettingsFrame:SetPoint("TOPLEFT", this, "BOTTOMLEFT");
  fontSettingsFrame:Raise();

  -- Make sure the event type is set for the button.
  if (this:GetParent().EventType ~= nil) then
   getglobal(FONT_SETTINGS_FRAME_NAME .. "Title"):SetText(this:GetParent().FontFrameTitle);
   MikSBTOpt.ShowEventFontSettings("Notification", this:GetParent().EventType);
  end
 end
end

-- **********************************************************************************
-- This function will enable and disable the load and delete profile
-- buttons according to which profile is selected.
-- **********************************************************************************
function MikSBTOpt.EnableDisableProfileButtons()
 local profileDropdown = getglobal(interfaceData.Dropdowns[DROPDOWN_INDEX_PROFILE].FrameName);
 local deleteProfileButton = getglobal(interfaceData.Buttons[BUTTON_INDEX_DELETE_PROFILE].FrameName);
 local loadProfileButton = getglobal(interfaceData.Buttons[BUTTON_INDEX_LOAD_PROFILE].FrameName);

 -- Enable the profile button manipulation buttons.
 deleteProfileButton:Enable();
 loadProfileButton:Enable();

 -- Check if the default profile is selected and disable the delete profile button.
 if (UIDropDownMenu_GetSelectedValue(profileDropdown) == MikSBT.DEFAULT_PROFILE_NAME) then
  deleteProfileButton:Disable();
 end 

 -- Check if it's the currently loaded profile and disable the load button.
 if (UIDropDownMenu_GetSelectedValue(profileDropdown) == MikSBT_Save.CurrentProfile) then
  loadProfileButton:Disable();
 end
end


-------------------------------------------------------------------------------------
-- Editbox Functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up all the editboxes.
-- **********************************************************************************
function  MikSBTOpt.SetupEditboxes()
 -- Setup the editbox interface data.
 MikSBTOpt.SetupInterfaceEditboxData();

 -- Setup the new profile name editbox.
 MikSBTOpt.SetupEditbox(EDITBOX_INDEX_NEW_PROFILE_NAME);
end


-- **********************************************************************************
-- Sets up the editbox for the passed data index.
-- **********************************************************************************
function MikSBTOpt.SetupEditbox(editboxDataIndex)
 local editboxData = interfaceData.Editboxes[editboxDataIndex];

 -- Make sure there is data for the editbox.
 if (editboxData) then
  local associatedFrame = getglobal(editboxData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   associatedFrame.editboxDataIndex = editboxDataIndex;
   associatedFrame.associatedVarPath = editboxData.VarPath;
   associatedFrame.tooltipText = editboxData.Tooltip;

   -- Set the label for the editbox.
   getglobal(editboxData.FrameName .. "Label"):SetText(editboxData.Label);

   -- Sets the default value.
   MikSBTOpt.PopulateEditbox(editboxDataIndex)
  end
 end
end


-- **********************************************************************************
-- Sets the editbox's text for the passed data index.  If a text value is passed
-- it is used, otherwise the function will attempt to get the text from the
-- VarPath field of the editbox data.
-- **********************************************************************************
function MikSBTOpt.PopulateEditbox(editboxDataIndex, editboxValue)
 local editboxData = interfaceData.Editboxes[editboxDataIndex];

 -- Make sure there is data for the editbox.
 if (editboxData) then
  local associatedFrame = getglobal(editboxData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   local tempValue = editboxValue;

   -- Check if no value was passed.
   if (tempValue == nil) then
    -- Check if there is an associated variable path for the editbox.
    if (editboxData.VarPath) then
     tempValue = MikSBT.GetOptionFromVarPath(editboxData.VarPath);
    end
   end

   -- Make sure there is a value to set.
   if (tempValue) then
    -- Set the value.
    associatedFrame:SetText(tempValue);
   end
  end
 end
end


-- **********************************************************************************
-- Called when the text changes in an edit box.
-- **********************************************************************************
function MikSBTOpt.EditboxOnTextChanged()
 -- New Profile Name
 if (this.editboxDataIndex == EDITBOX_INDEX_NEW_PROFILE_NAME) then
  local createProfileButton = getglobal(interfaceData.Buttons[BUTTON_INDEX_CREATE_PROFILE].FrameName);

  -- Enable the create profile button.
  createProfileButton:Enable();

  -- Check if there is no text and disable the create profile button.
  if (this:GetText() == "") then 
   createProfileButton:Disable();
  -- Check if the profile already exists.
  elseif (MikSBT.ProfileExists(this:GetText())) then
   createProfileButton:Disable();
  end
 else
  -- Check if there is an associated variable path and set it to the correct value.
  if (this.associatedVarPath) then
   -- Set the associated variable in the current profile to the value.
   MikSBT.SetOptionFromVarPath(this.associatedVarPath, this:GetText());
  end
 end

end


-------------------------------------------------------------------------------------
-- Color swatch functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Sets up the color swatch for the passed data index.
-- **********************************************************************************
function MikSBTOpt.SetupColorSwatch(colorSwatchDataIndex)
 local colorSwatchData = interfaceData.ColorSwatches[colorSwatchDataIndex];

 -- Make sure there is data for the color swatch.
 if (colorSwatchData) then
  local associatedFrame = getglobal(colorSwatchData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   associatedFrame.colorSwatchDataIndex = colorSwatchDataIndex;
   associatedFrame.associatedVarPath = colorSwatchData.VarPath;
   associatedFrame.tooltipText = colorSwatchData.Tooltip;

   -- Sets the default value.
   MikSBTOpt.PopulateColorSwatch(colorSwatchDataIndex);
  end
 end
end


-- **********************************************************************************
-- Sets the color swatch's color for the passed data index.  If color values are
-- passed they are used, otherwise the function will attempt to get the color values
-- from the VarPath field of the color swatch data.
-- **********************************************************************************
function MikSBTOpt.PopulateColorSwatch(colorSwatchDataIndex, colorR, colorG, colorB)
 local colorSwatchData = interfaceData.ColorSwatches[colorSwatchDataIndex];

 -- Make sure there is data for the color swatch.
 if (colorSwatchData) then
  local associatedFrame = getglobal(colorSwatchData.FrameName);

  -- Make sure the associated frame exists.
  if (associatedFrame) then
   local tempR = colorR;
   local tempG = colorG;
   local tempB = colorB;

   -- Check if no r value was passed.
   if (tempR == nil) then
    -- Check if there is an associated variable path for the color swatch.
    if (colorSwatchData.VarPath) then
     tempR = MikSBT.GetOptionFromVarPath(colorSwatchData.VarPath .. ".r");
    end
   end

   -- Check if no g value was passed.
   if (tempG == nil) then
    -- Check if there is an associated variable path for the color swatch.
    if (colorSwatchData.VarPath) then
     tempG = MikSBT.GetOptionFromVarPath(colorSwatchData.VarPath .. ".g");
    end
   end

   -- Check if no b value was passed.
   if (tempB == nil) then
    -- Check if there is an associated variable path for the color swatch.
    if (colorSwatchData.VarPath) then
     tempB = MikSBT.GetOptionFromVarPath(colorSwatchData.VarPath .. ".b");
    end
   end

   -- Make sure there are values to set.
   if (tempR ~= nil and tempG ~= nil and tempB ~= nil) then
    -- Set the values.
    local objColorSwatchTexture = getglobal(associatedFrame:GetName() .. "NormalTexture");
    objColorSwatchTexture:SetVertexColor(tempR, tempG, tempB);
   end
  end
 end
end


-- **********************************************************************************
-- Called when on of the color swatches for an event is clicked.
-- **********************************************************************************
function MikSBTOpt.ColorSwatchOnClick()
 -- Check if the color swatch has an associated variable path.
 if (this.associatedVarPath) then
  -- Hide the event message editbox if it's shown.
  MikSBTOpt.HideEventMessageEditbox(this:GetParent():GetParent():GetName().. "MessageEditbox");
  
  -- Get the r, g, and b values from the associated variable path and show the color picker.
  this.r = MikSBT.GetOptionFromVarPath(this.associatedVarPath .. ".r");
  this.g = MikSBT.GetOptionFromVarPath(this.associatedVarPath .. ".g");
  this.b = MikSBT.GetOptionFromVarPath(this.associatedVarPath .. ".b");
  this.swatchFunc = MikSBTOpt.ColorSwatchSetColor;
  this.cancelFunc = MikSBTOpt.ColorSwatchCancel;
  ColorPickerFrame.associatedVarPath = this.associatedVarPath;
  ColorPickerFrame.associatedColorSwatch = this;
  UIDropDownMenuButton_OpenColorPicker(this);
 end
end


-- **********************************************************************************
-- Caled when a color is selected for an event in the color picker dialog.
-- **********************************************************************************
function MikSBTOpt.ColorSwatchSetColor()
 -- Get the red, green, and blue color values.
 local r, g, b = ColorPickerFrame:GetColorRGB();

 -- Make sure there is an associated color swatch.
 if (ColorPickerFrame.associatedColorSwatch) then
  -- Get the associated color swatch texture and make sure it's valid.
  local objColorSwatchTexture = getglobal(ColorPickerFrame.associatedColorSwatch:GetName() .. "NormalTexture");
  if (objColorSwatchTexture) then
   -- Set the associated 
   objColorSwatchTexture:SetVertexColor(r, g, b);
  end
 end


 -- Check if the color swatch has an associated variable path.
 if (ColorPickerFrame.associatedVarPath) then
  -- Update current profile data.
  MikSBT.SetOptionFromVarPath(ColorPickerFrame.associatedVarPath .. ".r", r);
  MikSBT.SetOptionFromVarPath(ColorPickerFrame.associatedVarPath .. ".g", g);
  MikSBT.SetOptionFromVarPath(ColorPickerFrame.associatedVarPath .. ".b", b);
 end 
end


-- **********************************************************************************
-- Called when the color picker dialog is canceled.
-- **********************************************************************************
function MikSBTOpt.ColorSwatchCancel(previousValues)
 -- Make sure there is an associated color swatch.
 if (ColorPickerFrame.associatedColorSwatch) then
  -- Get the associated color swatch texture and make sure it's valid.
  local objColorSwatchTexture = getglobal(ColorPickerFrame.associatedColorSwatch:GetName() .. "NormalTexture");
  if (objColorSwatchTexture) then
   -- Set the associated 
   objColorSwatchTexture:SetVertexColor(previousValues.r, previousValues.g, previousValues.b);
  end
 end

 -- Check if the color swatch has an associated variable path.
 if (ColorPickerFrame.associatedVarPath) then
  -- Update current profile data.
  MikSBT.SetOptionFromVarPath(ColorPickerFrame.associatedVarPath .. ".r", previousValues.r);
  MikSBT.SetOptionFromVarPath(ColorPickerFrame.associatedVarPath .. ".g", previousValues.g);
  MikSBT.SetOptionFromVarPath(ColorPickerFrame.associatedVarPath .. ".b", previousValues.b);
 end
end


-------------------------------------------------------------------------------------
-- Event Settings functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- This function populates the incoming event settings.
-- **********************************************************************************
function MikSBTOpt.PopulateIncomingEventSettings()
 -- Hide the event message editbox if it's shown.
 MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab2FrameMessageEditbox");

 -- Get the total number of events.
 totalItems = table.getn(MikSBTOpt.INCOMING_EVENTS);

 -- Get the incoming event settings scroll frame and its offset.
 local objFrame = getglobal(OPTIONS_FRAME_NAME .. "Tab2FrameEventsScrollFrame");
 FauxScrollFrame_Update(objFrame, totalItems, EVENT_ITEMS_PER_PAGE, EVENT_ITEM_HEIGHT);
 local skipCount = FauxScrollFrame_GetOffset(objFrame);

 local objEventSettings = nil;
 -- Loop through all of the visible items.
 for i = 1, EVENT_ITEMS_PER_PAGE do
  objEventSettings = getglobal(OPTIONS_FRAME_NAME .. "Tab2FrameEvent" .. i);

  -- Make sure the event settings object exists.
  if (objEventSettings) then
   -- Check if there is not event settings data for the event settings slot.
   if (i > totalItems) then
    -- Hide the event settings.
    objEventSettings:Hide();

   -- There is data for the event settings slot.
   else

    -- Make sure there is an associated event type and set it.
    if (interfaceData.IncomingEvents[i+skipCount].EventType ~= nil) then
     objEventSettings.EventType = interfaceData.IncomingEvents[i+skipCount].EventType;
    end

    -- Set the font frame title to the checkbox's label.
    objEventSettings.FontFrameTitle = interfaceData.IncomingEvents[i+skipCount].Label;


    -- Change the interface data for the event setting color swatch and call the setup routine for it.
    local colorSwatchDataIndex = COLORSWATCH_INDEX_INCOMING_EVENTS_START + (i-1);
    local colorSwatchData = interfaceData.ColorSwatches[colorSwatchDataIndex];
    colorSwatchData.Tooltip = MikSBTOpt.MSG_EVENT_COLOR_SWATCH_TOOLTIP;
    colorSwatchData.FrameName = objEventSettings:GetName() .. "ColorSwatch";
    if (objEventSettings.EventType ~= nil) then
     colorSwatchData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".FontSettings.Color";
    end
    MikSBTOpt.SetupColorSwatch(colorSwatchDataIndex);
    

    -- Change the interface data for the event setting show checkbox and call the setup routine for it.
    local checkboxDataIndex = CHECKBOX_INDEX_INCOMING_EVENTS_SHOW_START + (i-1);
    local checkboxData = interfaceData.Checkboxes[checkboxDataIndex];
    checkboxData.Label = interfaceData.IncomingEvents[i+skipCount].Label;
    checkboxData.Tooltip = interfaceData.IncomingEvents[i+skipCount].Tooltips.ShowCheckbox;
    checkboxData.FrameName = objEventSettings:GetName() .. "ShowEventCheckbox";
    if (objEventSettings.EventType ~= nil) then
     checkboxData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".Show";
    end
    MikSBTOpt.SetupCheckbox(checkboxDataIndex);

    -- Set the message label to the appropriate text and the editbox's tooltip.
    local objMessageLabel = getglobal(objEventSettings:GetName() .. "MessageLabel");
    if (objMessageLabel) then
     -- Set the tooltip for the label.
     objMessageLabel:GetParent().tooltipText = MikSBTOpt.MSG_EVENT_MESSAGE_LABEL_TOOLTIP;
     if (objEventSettings.EventType ~= nil) then
      objMessageLabel:SetText(MikSBT.GetOptionFromVarPath("EventSettings." .. objEventSettings.EventType .. ".Message"));
     end
    end
    objEventSettings.EditboxTooltip = interfaceData.IncomingEvents[i+skipCount].Tooltips.Editbox;

    -- Hide the sticky event checkbox.
    getglobal(objEventSettings:GetName() .. "StickyEventCheckbox"):Hide();

    -- Change the interface data for the event setting edit font settings button and call the setup routine for it.
    local buttonDataIndex = BUTTON_INDEX_INCOMING_EVENTS_START + (i-1);
    local buttonData = interfaceData.Buttons[buttonDataIndex];
    buttonData.Label = MikSBTOpt.MSG_EVENT_EDIT_FONT_SETTINGS_BUTTON_LABEL;
    buttonData.Tooltip = MikSBTOpt.MSG_EVENT_EDIT_FONT_SETTINGS_BUTTON_TOOLTIP;
    buttonData.FrameName = objEventSettings:GetName() .. "EditFontSettingsButton";
    MikSBTOpt.SetupButton(buttonDataIndex);
    

    -- Show the event settings.
    objEventSettings:Show();
   end
  end
 end
 
end


-- **********************************************************************************
-- This function populates the outgoing event settings.
-- **********************************************************************************
function MikSBTOpt.PopulateOutgoingEventSettings()
 -- Hide the event message editbox if it's shown.
 MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab3FrameMessageEditbox");

 -- Get the total number of events.
 local totalItems = table.getn(MikSBTOpt.OUTGOING_EVENTS);

 local objFrame = getglobal(OPTIONS_FRAME_NAME .. "Tab3FrameEventsScrollFrame");
 FauxScrollFrame_Update(objFrame, totalItems, EVENT_ITEMS_PER_PAGE, EVENT_ITEM_HEIGHT);
 local skipCount = FauxScrollFrame_GetOffset(objFrame);

 local objEventSettings = nil;
 -- Loop through all of the visible items.
 for i = 1, EVENT_ITEMS_PER_PAGE do
  objEventSettings = getglobal(OPTIONS_FRAME_NAME .. "Tab3FrameEvent" .. i);

  -- Make sure the event settings object exists.
  if (objEventSettings) then
   -- Check if there is not event settings data for the event settings slot.
   if (i > totalItems) then
    -- Hide the event settings.
    objEventSettings:Hide();

   -- There is data for the event settings slot.
   else
    -- Make sure there is an associated event type and set it.
    if (interfaceData.OutgoingEvents[i+skipCount].EventType ~= nil) then
     objEventSettings.EventType = interfaceData.OutgoingEvents[i+skipCount].EventType;
    end

    -- Set the font frame title to the checkbox's label.
    objEventSettings.FontFrameTitle = interfaceData.OutgoingEvents[i+skipCount].Label;


    -- Change the interface data for the event setting color swatch and call the setup routine for it.
    local colorSwatchDataIndex = COLORSWATCH_INDEX_OUTGOING_EVENTS_START + (i-1);
    local colorSwatchData = interfaceData.ColorSwatches[colorSwatchDataIndex];
    colorSwatchData.Tooltip = MikSBTOpt.MSG_EVENT_COLOR_SWATCH_TOOLTIP;
    colorSwatchData.FrameName = objEventSettings:GetName() .. "ColorSwatch";
    if (objEventSettings.EventType ~= nil) then
     colorSwatchData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".FontSettings.Color";
    end
    MikSBTOpt.SetupColorSwatch(colorSwatchDataIndex);
    

    -- Change the interface data for the event setting show checkbox and call the setup routine for it.
    local checkboxDataIndex = CHECKBOX_INDEX_OUTGOING_EVENTS_SHOW_START + (i-1);
    local checkboxData = interfaceData.Checkboxes[checkboxDataIndex];
    checkboxData.Label = interfaceData.OutgoingEvents[i+skipCount].Label;
    checkboxData.Tooltip = interfaceData.OutgoingEvents[i+skipCount].Tooltips.ShowCheckbox;
    checkboxData.FrameName = objEventSettings:GetName() .. "ShowEventCheckbox";
    if (objEventSettings.EventType ~= nil) then
     checkboxData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".Show";
    end
    MikSBTOpt.SetupCheckbox(checkboxDataIndex);

    -- Set the message label to the appropriate text and the editbox's tooltip.
    local objMessageLabel = getglobal(objEventSettings:GetName() .. "MessageLabel");
    if (objMessageLabel) then
     -- Set the tooltip for the label.
     objMessageLabel:GetParent().tooltipText = MikSBTOpt.MSG_EVENT_MESSAGE_LABEL_TOOLTIP;
     if (objEventSettings.EventType ~= nil) then
      objMessageLabel:SetText(MikSBT.GetOptionFromVarPath("EventSettings." .. objEventSettings.EventType .. ".Message"));
     end
    end
    objEventSettings.EditboxTooltip = interfaceData.OutgoingEvents[i+skipCount].Tooltips.Editbox;

    -- Hide the sticky event checkbox.
    getglobal(objEventSettings:GetName() .. "StickyEventCheckbox"):Hide();

    -- Change the interface data for the event setting edit font settings button and call the setup routine for it.
    local buttonDataIndex = BUTTON_INDEX_OUTGOING_EVENTS_START + (i-1);
    local buttonData = interfaceData.Buttons[buttonDataIndex];
    buttonData.Label = MikSBTOpt.MSG_EVENT_EDIT_FONT_SETTINGS_BUTTON_LABEL;
    buttonData.Tooltip = MikSBTOpt.MSG_EVENT_EDIT_FONT_SETTINGS_BUTTON_TOOLTIP;
    buttonData.FrameName = objEventSettings:GetName() .. "EditFontSettingsButton";
    MikSBTOpt.SetupButton(buttonDataIndex);

   end
  end
 end
end


-- **********************************************************************************
-- This function populates the notification event settings.
-- **********************************************************************************
function MikSBTOpt.PopulateNotificationEventSettings()
 -- Hide the event message editbox if it's shown.
 MikSBTOpt.HideEventMessageEditbox(OPTIONS_FRAME_NAME .. "Tab4FrameMessageEditbox");

 -- Get the total number of events.
 local totalItems = table.getn(MikSBTOpt.NOTIFICATION_EVENTS);

 local objFrame = getglobal(OPTIONS_FRAME_NAME .. "Tab4FrameEventsScrollFrame");
 FauxScrollFrame_Update(objFrame, totalItems, EVENT_ITEMS_PER_PAGE, EVENT_ITEM_HEIGHT);
 local skipCount = FauxScrollFrame_GetOffset(objFrame);

 local objEventSettings = nil;
 -- Loop through all of the visible items.
 for i = 1, EVENT_ITEMS_PER_PAGE do
  objEventSettings = getglobal(OPTIONS_FRAME_NAME .. "Tab4FrameEvent" .. i);

  -- Make sure the event settings object exists.
  if (objEventSettings) then
   -- Check if there is not event settings data for the event settings slot.
   if (i > totalItems) then
    -- Hide the event settings.
    objEventSettings:Hide();

   -- There is data for the event settings slot.
   else
    -- Make sure there is an associated event type and set it.
    if (interfaceData.NotificationEvents[i+skipCount].EventType ~= nil) then
     objEventSettings.EventType = interfaceData.NotificationEvents[i+skipCount].EventType;
    end

    -- Set the font frame title to the checkbox's label.
    objEventSettings.FontFrameTitle = interfaceData.NotificationEvents[i+skipCount].Label;

    -- Change the interface data for the event setting color swatch and call the setup routine for it.
    local colorSwatchDataIndex = COLORSWATCH_INDEX_NOTIFICATION_EVENTS_START + (i-1);
    local colorSwatchData = interfaceData.ColorSwatches[colorSwatchDataIndex];
    colorSwatchData.Tooltip = MikSBTOpt.MSG_EVENT_COLOR_SWATCH_TOOLTIP;
    colorSwatchData.FrameName = objEventSettings:GetName() .. "ColorSwatch";
    if (objEventSettings.EventType ~= nil) then
     colorSwatchData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".FontSettings.Color";
    end
    MikSBTOpt.SetupColorSwatch(colorSwatchDataIndex);
    

    -- Change the interface data for the event setting show checkbox and call the setup routine for it.
    local checkboxDataIndex = CHECKBOX_INDEX_OUTGOING_EVENTS_SHOW_START + (i-1);
    local checkboxData = interfaceData.Checkboxes[checkboxDataIndex];
    checkboxData.Label = interfaceData.NotificationEvents[i+skipCount].Label;
    checkboxData.Tooltip = interfaceData.NotificationEvents[i+skipCount].Tooltips.ShowCheckbox;
    checkboxData.FrameName = objEventSettings:GetName() .. "ShowEventCheckbox";
    if (objEventSettings.EventType ~= nil) then
     checkboxData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".Show";
    end
    MikSBTOpt.SetupCheckbox(checkboxDataIndex);

    -- Set the message label to the appropriate text and the editbox's tooltip.
    local objMessageLabel = getglobal(objEventSettings:GetName() .. "MessageLabel");
    if (objMessageLabel) then
     -- Set the tooltip for the label.
     objMessageLabel:GetParent().tooltipText = MikSBTOpt.MSG_EVENT_MESSAGE_LABEL_TOOLTIP;
     if (objEventSettings.EventType ~= nil) then
      objMessageLabel:SetText(MikSBT.GetOptionFromVarPath("EventSettings." .. objEventSettings.EventType .. ".Message"));
     end
    end
    objEventSettings.EditboxTooltip = interfaceData.NotificationEvents[i+skipCount].Tooltips.Editbox;


    -- Change the interface data for the event setting sticky checkbox and call the setup routine for it.
    checkboxDataIndex = CHECKBOX_INDEX_NOTIFICATION_EVENTS_STICKY_START + (i-1);
    checkboxData = interfaceData.Checkboxes[checkboxDataIndex];
    checkboxData.Tooltip = MikSBTOpt.MSG_EVENT_STICKY_CHECKBOX_TOOLTIP;
    checkboxData.FrameName = objEventSettings:GetName() .. "StickyEventCheckbox";
    if (objEventSettings.EventType ~= nil) then
     checkboxData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".IsSticky";
    end
    MikSBTOpt.SetupCheckbox(checkboxDataIndex);
    getglobal(checkboxData.FrameName):Show();



    -- Change the interface data for the event setting edit font settings button and call the setup routine for it.
    local buttonDataIndex = BUTTON_INDEX_NOTIFICATION_EVENTS_START + (i-1);
    local buttonData = interfaceData.Buttons[buttonDataIndex];
    buttonData.Label = MikSBTOpt.MSG_EVENT_EDIT_FONT_SETTINGS_BUTTON_LABEL;
    buttonData.Tooltip = MikSBTOpt.MSG_EVENT_EDIT_FONT_SETTINGS_BUTTON_TOOLTIP;
    buttonData.FrameName = objEventSettings:GetName() .. "EditFontSettingsButton";
    MikSBTOpt.SetupButton(buttonDataIndex);
   end
  end
 end
end


-- **********************************************************************************
-- Called when one of the event message labels is clicked.
-- **********************************************************************************
function MikSBTOpt.EventMessageLabelOnClick()
 -- Get the event settings frame.
 local objEventSettings = this:GetParent();

 -- Make sure the event type is set for the label and there is an event data index.
 if (objEventSettings.EventType ~= nil) then
  -- Get the clicked label and hide it.
  local labelFrameName = this:GetName() .. "Label";
  local objLabel = getglobal(labelFrameName);
  objLabel:Hide();

  -- Get the message editbox for the scroll area.
  local editboxFrameName = objEventSettings:GetParent():GetName() .. "MessageEditbox";
  local objEditbox = getglobal(editboxFrameName);

  -- Hide the editbox to make sure everything is updated correctly.
  objEditbox:Hide();

  --  Show the editbox where the clicked label is, set the clicked label frame name for the editbox, and
  -- set the previous text to the label's text.
  objEditbox:Show();
  objEditbox:ClearAllPoints();
  objEditbox:SetPoint("TOPLEFT", this, "TOPLEFT");
  objEditbox.ClickedLabelFrameName = labelFrameName;
  objEditbox.PreviousText = objLabel:GetText();

  -- Change the interface data for the event setting editbox and call the setup routine for it.
  local editboxDataIndex = EDITBOX_INDEX_EVENT_MESSAGE;
  local editboxData = interfaceData.Editboxes[editboxDataIndex];
  if (objEventSettings.EditboxTooltip ~= nil) then
   editboxData.Tooltip = MikSBTOpt.MSG_EVENT_MESSAGE_EDITBOX_TOOLTIP_PREFIX .. objEventSettings.EditboxTooltip;
  end
  editboxData.FrameName = editboxFrameName;
  editboxData.VarPath = "EventSettings." .. objEventSettings.EventType .. ".Message";
  MikSBTOpt.SetupEditbox(editboxDataIndex);  

  -- Set the focus to the editbox.
  objEditbox:SetFocus();
 end
end


-- **********************************************************************************
-- Hides the event message editbox.
-- **********************************************************************************
function MikSBTOpt.HideEventMessageEditbox(editboxFrameName)
 -- Check if the event message editbox is shown and hide it.
 local objEditbox = getglobal(editboxFrameName);
 if (objEditbox and objEditbox:IsVisible()) then
  objEditbox:Hide();
 end
end

-------------------------------------------------------------------------------------
-- Font Settings Functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Show master font settings.
-- **********************************************************************************
function MikSBTOpt.ShowMasterFontSettings()
 -- Show the crit font settings controls.
 MikSBTOpt.ShowCritFontControls();


 -- Setup the normal font dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "MasterFontSettings.Normal.FontIndex";
 ifData.Options = MikSBT.AVAILABLE_FONTS;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Master"].Font;
 MikSBTOpt.SetupDropdown(dataIndex);

 -- Setup the normal font outline dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT_OUTLINE;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "MasterFontSettings.Normal.OutlineIndex";
 ifData.Options = MikSBT.AVAILABLE_OUTLINES;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Master"].Outline;
 MikSBTOpt.SetupDropdown(dataIndex);


 -- Setup the crit font dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT_CRIT;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "MasterFontSettings.Crit.FontIndex";
 ifData.Options = MikSBT.AVAILABLE_FONTS;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Master"].FontCrit;
 MikSBTOpt.SetupDropdown(dataIndex);

 -- Setup the crit font outline dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT_OUTLINE_CRIT;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "MasterFontSettings.Crit.OutlineIndex";
 ifData.Options = MikSBT.AVAILABLE_OUTLINES;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Master"].OutlineCrit;
 MikSBTOpt.SetupDropdown(dataIndex);


 -- Setup the normal font size slider.
 local dataIndex = SLIDER_INDEX_FONT_SIZE;
 local ifData = interfaceData.Sliders[dataIndex];
 ifData.VarPath = "MasterFontSettings.Normal.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Master"].FontSize;
 MikSBTOpt.SetupSlider(dataIndex);

 -- Setup the crit font size slider.
 local dataIndex = SLIDER_INDEX_FONT_SIZE_CRIT;
 local ifData = interfaceData.Sliders[dataIndex];
 ifData.VarPath = "MasterFontSettings.Crit.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Master"].FontSizeCrit;
 MikSBTOpt.SetupSlider(dataIndex);

 -- Hide the inherit font size and crit font size checkboxes.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox"):Hide();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):Hide();
 
 -- Show the font size and crit size sliders.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider"):Show();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider"):Show();


 -- Show the font settings frame.
 getglobal(FONT_SETTINGS_FRAME_NAME):Show();

 -- Update the font previews.
 MikSBTOpt.UpdateFontPreviews();
end


-- **********************************************************************************
-- Show scroll area font settings .
-- **********************************************************************************
function MikSBTOpt.ShowScrollAreaFontSettings(scrollArea)
 -- Get the font settings frame.
 local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);

 -- Get the inherited font settings.
 fontSettingsFrame.InheritedFontIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Normal.FontIndex");
 fontSettingsFrame.InheritedOutlineIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Normal.OutlineIndex");
 fontSettingsFrame.InheritedFontSize = MikSBT.GetOptionFromVarPath("MasterFontSettings.Normal.FontSize");
 fontSettingsFrame.InheritedFontCritIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Crit.FontIndex");
 fontSettingsFrame.InheritedOutlineCritIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Crit.OutlineIndex");
 fontSettingsFrame.InheritedFontSizeCrit = MikSBT.GetOptionFromVarPath("MasterFontSettings.Crit.FontSize");


 -- Show the crit font settings controls.
 MikSBTOpt.ShowCritFontControls();


 -- Setup the normal font dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Normal.FontIndex";
 ifData.Options = availableFontsInherit;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].Font;
 MikSBTOpt.SetupDropdown(dataIndex);

 -- Setup the normal font outline dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT_OUTLINE;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Normal.OutlineIndex";
 ifData.Options = availableOutlinesInherit;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].Outline;
 MikSBTOpt.SetupDropdown(dataIndex);

 -- Setup the crit font dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT_CRIT;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Crit.FontIndex";
 ifData.Options = availableFontsInherit;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].FontCrit;
 MikSBTOpt.SetupDropdown(dataIndex);

 -- Setup the crit font outline dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT_OUTLINE_CRIT;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Crit.OutlineIndex";
 ifData.Options = availableOutlinesInherit;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].OutlineCrit;
 MikSBTOpt.SetupDropdown(dataIndex);


 -- Show the inherit font size and crit font size checkboxes.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox"):Show();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):Show();
 
 -- Setup the inherit normal font size checkbox.
 local dataIndex = CHECKBOX_INDEX_INHERIT_FONT_SIZE;
 local ifData = interfaceData.Checkboxes[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Normal.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].InheritFontSize;
 MikSBTOpt.SetupCheckbox(dataIndex);

 -- Setup the inherit crit font size checkbox.
 local dataIndex = CHECKBOX_INDEX_INHERIT_FONT_SIZE_CRIT;
 local ifData = interfaceData.Checkboxes[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Crit.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].InheritFontSizeCrit;
 MikSBTOpt.SetupCheckbox(dataIndex);


 -- Change the normal font size slider interface data.
 local dataIndex = SLIDER_INDEX_FONT_SIZE;
 local ifData = interfaceData.Sliders[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Normal.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].FontSize;

 -- Change the crit font size slider interface data.
 local dataIndex = SLIDER_INDEX_FONT_SIZE_CRIT;
 local ifData = interfaceData.Sliders[dataIndex];
 ifData.VarPath = "DisplaySettings." .. scrollArea .. ".FontSettings.Crit.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["ScrollArea"].FontSizeCrit;



 -- Check if the normal font size is inherited.
 if (MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Normal.FontSize") == 0) then
  -- Check the inherited checkbox and hide the slider.
  getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox"):SetChecked(true);
  getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider"):Hide();
 else
  -- Uncheck the inherited checkbox and setup the slider.
  getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox"):SetChecked(false);
  getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider"):Show();

  -- Setup the normal font size slider.
  MikSBTOpt.SetupSlider(SLIDER_INDEX_FONT_SIZE);
 end


 -- Check if the critical font size is inherited.
 if (MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Crit.FontSize") == 0) then
  -- Check the inherited checkbox and hide the slider.
  getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):SetChecked(true);
  getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider"):Hide();
 else
  -- Uncheck the inherited checkbox and setup the slider.
  getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):SetChecked(false);
  getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider"):Show();

  -- Setup the crit font size slider.
  MikSBTOpt.SetupSlider(SLIDER_INDEX_FONT_SIZE_CRIT);
 end


 -- Show the font settings frame.
 fontSettingsFrame:Show();

 -- Update the font previews.
 MikSBTOpt.UpdateFontPreviews();
end


-- **********************************************************************************
-- Show event font settings.
-- **********************************************************************************
function MikSBTOpt.ShowEventFontSettings(scrollArea, eventType)
 -- Get the inherited font index.
 local inheritedFontIndex = MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Normal.FontIndex");
 if (inheritedFontIndex == 0) then
  inheritedFontIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Normal.FontIndex");
 end

 -- Get the inherited outline index.
 local inheritedOutlineIndex = MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Normal.OutlineIndex");
 if (inheritedOutlineIndex == 0) then
  inheritedOutlineIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Normal.OutlineIndex");
 end
 
 -- Get the inherited font size.
 local inheritedFontSize = MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Normal.FontSize");
 if (inheritedFontSize == 0) then
  -- Get the font size from the scroll area and check if it's inherited from the master font size.
  inheritedFontSize = MikSBT.GetOptionFromVarPath("MasterFontSettings.Normal.FontSize");
 end



 -- Get the inherited crit font index.
 local inheritedFontCritIndex = MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Crit.FontIndex");
 if (inheritedFontCritIndex == 0) then
  inheritedFontCritIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Crit.FontIndex");
 end

 -- Get the inherited crit outline index.
 local inheritedOutlineCritIndex = MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Crit.OutlineIndex");
 if (inheritedOutlineCritIndex == 0) then
  inheritedOutlineCritIndex = MikSBT.GetOptionFromVarPath("MasterFontSettings.Crit.OutlineIndex");
 end
 
 -- Get the inherited crit font size.
 local inheritedFontSizeCrit = MikSBT.GetOptionFromVarPath("DisplaySettings." .. scrollArea .. ".FontSettings.Crit.FontSize");
 if (inheritedFontSizeCrit == 0) then
  -- Get the font size from the scroll area and check if it's inherited from the master font size.
  inheritedFontSizeCrit = MikSBT.GetOptionFromVarPath("MasterFontSettings.Crit.FontSize");
 end


 -- Get the font settings frame.
 local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);


 -- Set the inherited settings to the frame.
 fontSettingsFrame.InheritedFontIndex = inheritedFontIndex;
 fontSettingsFrame.InheritedOutlineIndex = inheritedOutlineIndex;
 fontSettingsFrame.InheritedFontSize = inheritedFontSize;
 fontSettingsFrame.InheritedFontCritIndex = inheritedFontCritIndex;
 fontSettingsFrame.InheritedOutlineCritIndex = inheritedOutlineCritIndex;
 fontSettingsFrame.InheritedFontSizeCrit = inheritedFontSizeCrit;


 -- Setup the normal font dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Normal.FontIndex";
 ifData.Options = availableFontsInherit;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].Font;
 MikSBTOpt.SetupDropdown(dataIndex);

 -- Setup the normal font outline dropdown.
 local dataIndex = DROPDOWN_INDEX_FONT_OUTLINE;
 local ifData = interfaceData.Dropdowns[dataIndex];
 ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Normal.OutlineIndex";
 ifData.Options = availableOutlinesInherit;
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].Outline;
 MikSBTOpt.SetupDropdown(dataIndex);


 -- Show the inherit normal font size checkbox.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox"):Show();

 -- Setup the inherit normal font size checkbox.
 local dataIndex = CHECKBOX_INDEX_INHERIT_FONT_SIZE;
 local ifData = interfaceData.Checkboxes[dataIndex];
 ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Normal.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].InheritFontSize;
 MikSBTOpt.SetupCheckbox(dataIndex);

 -- Change the normal font size slider interface data.
 local dataIndex = SLIDER_INDEX_FONT_SIZE;
 local ifData = interfaceData.Sliders[dataIndex];
 ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Normal.FontSize";
 ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].FontSize;

 -- Check if the normal font size is inherited.
 if (MikSBT.GetOptionFromVarPath("EventSettings." .. eventType .. ".FontSettings.Normal.FontSize") == 0) then
  -- Check the inherited checkbox and hide the slider.
  getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox"):SetChecked(true);
  getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider"):Hide();
 else
  -- Uncheck the inherited checkbox and setup the slider.
  getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCheckbox"):SetChecked(false);
  getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider"):Show();

  -- Setup the normal font size slider.
  MikSBTOpt.SetupSlider(SLIDER_INDEX_FONT_SIZE);
 end


 -- Check if the event type supports crits.
 if (MikSBT_Save.Profiles[MikSBT_Save.CurrentProfile].EventSettings[eventType].FontSettings.Crit ~= nil) then
  -- Show the crit font settings controls.
  MikSBTOpt.ShowCritFontControls();


  -- Setup the crit font dropdown.
  local dataIndex = DROPDOWN_INDEX_FONT_CRIT;
  local ifData = interfaceData.Dropdowns[dataIndex];
  ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Crit.FontIndex";
  ifData.Options = availableFontsInherit;
  ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].FontCrit;
  MikSBTOpt.SetupDropdown(dataIndex);

  -- Setup the crit font outline dropdown.
  local dataIndex = DROPDOWN_INDEX_FONT_OUTLINE_CRIT;
  local ifData = interfaceData.Dropdowns[dataIndex];
  ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Crit.OutlineIndex";
  ifData.Options = availableOutlinesInherit;
  ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].OutlineCrit;
  MikSBTOpt.SetupDropdown(dataIndex);


  -- Show the inherit crit font size checkbox.
  getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):Show();

  -- Setup the inherit crit font size checkbox.
  local dataIndex = CHECKBOX_INDEX_INHERIT_FONT_SIZE_CRIT;
  local ifData = interfaceData.Checkboxes[dataIndex];
  ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Crit.FontSize";
  ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].InheritFontSizeCrit;
  MikSBTOpt.SetupCheckbox(dataIndex);

  -- Change the crit font size slider interface data.
  local dataIndex = SLIDER_INDEX_FONT_SIZE_CRIT;
  local ifData = interfaceData.Sliders[dataIndex];
  ifData.VarPath = "EventSettings." .. eventType .. ".FontSettings.Crit.FontSize";
  ifData.Tooltip = MikSBTOpt.FONT_SETTINGS_TOOLTIPS["Event"].FontSizeCrit;

  -- Check if the critical font size is inherited.
  if (MikSBT.GetOptionFromVarPath("EventSettings." .. eventType .. ".FontSettings.Crit.FontSize") == 0) then
   -- Check the inherited checkbox and hide the slider.
   getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):SetChecked(true);
   getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider"):Hide();
  else
   -- Uncheck the inherited checkbox and setup the slider.
   getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):SetChecked(false);
   getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider"):Show();

   -- Setup the crit font size slider.
   MikSBTOpt.SetupSlider(SLIDER_INDEX_FONT_SIZE_CRIT);
  end

 -- The event type does not support crits.
 else
  -- Hide the crit font settings controls.
  MikSBTOpt.HideCritFontControls();
 end


 -- Show the font settings frame.
 fontSettingsFrame:Show();

 -- Update the font previews.
 MikSBTOpt.UpdateFontPreviews();
end


-- **********************************************************************************
-- Updates the font previews with values selected.
-- **********************************************************************************
function MikSBTOpt.UpdateFontPreviews()
 -- Get the font settings frame.
 local fontSettingsFrame = getglobal(FONT_SETTINGS_FRAME_NAME);

 -- Get the selected normal font index.
 local fontIndex = UIDropDownMenu_GetSelectedValue(getglobal(FONT_SETTINGS_FRAME_NAME .. "FontDropdown"));
 if (fontIndex == 0) then
  fontIndex = fontSettingsFrame.InheritedFontIndex;
 end

 -- Get the selected normal outline index.
 local outlineIndex = UIDropDownMenu_GetSelectedValue(getglobal(FONT_SETTINGS_FRAME_NAME .. "FontOutlineDropdown"));
 if (outlineIndex == 0) then
  outlineIndex = fontSettingsFrame.InheritedOutlineIndex;
 end

 -- Check if the normal font size slider is shown.
 local slider = getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeSlider");
 local fontSize;
 if (slider:IsShown()) then
  -- Get the selected font size.
  fontSize = slider:GetValue();
 else
  -- Get the inherited font size.
  fontSize = fontSettingsFrame.InheritedFontSize;
 end


 -- Make sure the normal font index is valid.
 if (fontIndex ~= nil) then
  local previewText = getglobal(FONT_SETTINGS_FRAME_NAME .. "NormalPreviewText");
  previewText:SetFont(MikSBT.AVAILABLE_FONTS[fontIndex].Path, fontSize, MikSBT.AVAILABLE_OUTLINES[outlineIndex].Style);
 end



 -- Check if crits are to be shown.
 if (getglobal(FONT_SETTINGS_FRAME_NAME .. "CritPreview"):IsShown()) then
  -- Get the selected crit font index.
  local fontCritIndex = UIDropDownMenu_GetSelectedValue(getglobal(FONT_SETTINGS_FRAME_NAME .. "FontCritDropdown"));
  if (fontCritIndex == 0) then
   fontCritIndex = fontSettingsFrame.InheritedFontCritIndex;
  end


  -- Get the selected crit outline index.
  local outlineCritIndex = UIDropDownMenu_GetSelectedValue(getglobal(FONT_SETTINGS_FRAME_NAME .. "FontOutlineCritDropdown"));
  if (outlineCritIndex == 0) then
   outlineCritIndex = fontSettingsFrame.InheritedOutlineCritIndex;
  end


  -- Check if the crit font size slider is shown.
  local slider = getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider");
  local fontSizeCrit;
  if (slider:IsShown()) then
   -- Get the selected crit font size.
   fontSizeCrit = slider:GetValue();
  else
   -- Get the inherited crit font size.
   fontSizeCrit = fontSettingsFrame.InheritedFontSizeCrit;
  end

  -- Make sure the crit font index is valid.
  if (fontCritIndex ~= nil) then
   local previewText = getglobal(FONT_SETTINGS_FRAME_NAME .. "CritPreviewText");
   previewText:SetFont(MikSBT.AVAILABLE_FONTS[fontCritIndex].Path, fontSizeCrit, MikSBT.AVAILABLE_OUTLINES[outlineCritIndex].Style);
  end
 end
end


-- **********************************************************************************
-- Shows the crit font settings controls.
-- **********************************************************************************
function MikSBTOpt.ShowCritFontControls()
 -- Show all of the crit font settings controls.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontCritDropdown"):Show();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontOutlineCritDropdown"):Show();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):Show();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider"):Show();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "CritPreview"):Show();

 -- Move the normal font controls back to the left side.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontDropdown"):SetPoint("TOPLEFT", FONT_SETTINGS_FRAME_NAME, "TOPLEFT", 0, -60);
end


-- **********************************************************************************
-- Hides the crit font settings controls.
-- **********************************************************************************
function MikSBTOpt.HideCritFontControls()
 -- Hide all of the crit font settings controls.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontCritDropdown"):Hide();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontOutlineCritDropdown"):Hide();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "InheritFontSizeCritCheckbox"):Hide();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontSizeCritSlider"):Hide();
 getglobal(FONT_SETTINGS_FRAME_NAME .. "CritPreview"):Hide();

 -- Center the normal font.
 getglobal(FONT_SETTINGS_FRAME_NAME .. "FontDropdown"):SetPoint("TOPLEFT", FONT_SETTINGS_FRAME_NAME, "TOPLEFT", 85, -60);
end


-------------------------------------------------------------------------------------
-- Scroll Area Mover Functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Setup and show the scroll area mover.
-- **********************************************************************************
function MikSBTOpt.ShowScrollAreaMover()
 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  local scrollHeight = MikSBT.GetOptionFromVarPath(frame.ScrollHeightVarPath);
  frame.SavedXOffset = MikSBT.GetOptionFromVarPath(frame.XOffsetVarPath);
  frame.SavedYOffset = MikSBT.GetOptionFromVarPath(frame.YOffsetVarPath);

  frame:SetHeight(scrollHeight);
  frame:ClearAllPoints();
  frame:SetPoint("BOTTOM", UIParent, "CENTER", frame.SavedXOffset, frame.SavedYOffset);  

  frame:Show();

  -- Get the options frame and make sure it exists.
  local optionsFrame = getglobal(OPTIONS_FRAME_NAME);
  if (optionsFrame) then
   -- Hide the options frame.
   optionsFrame:Hide();
  end
 end
end


-- **********************************************************************************
-- Setup and show the incoming scroll area mover.
-- **********************************************************************************
function MikSBTOpt.ShowIncomingScrollAreaMover()
 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Set the variable paths for the x and y offsets.
  frame.XOffsetVarPath = "DisplaySettings.Incoming.FrameOffsets.x";
  frame.YOffsetVarPath = "DisplaySettings.Incoming.FrameOffsets.y";
  frame.ScrollHeightVarPath = "DisplaySettings.Incoming.ScrollHeight";
  frame.DisplayType = MikSBT.DISPLAYTYPE_INCOMING;

  -- Set the tooltip for the frame.
  frame.tooltipText = MikSBTOpt.FRAMES[FRAME_INDEX_INCOMING_SCROLL_AREA_MOVER].Tooltip;
 end

 -- Show the scroll area mover.
 MikSBTOpt.ShowScrollAreaMover();
end


-- **********************************************************************************
-- Setup and show the outgoing scroll area mover.
-- **********************************************************************************
function MikSBTOpt.ShowOutgoingScrollAreaMover()
 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Set the variable paths for the x and y offsets.
  frame.XOffsetVarPath = "DisplaySettings.Outgoing.FrameOffsets.x";
  frame.YOffsetVarPath = "DisplaySettings.Outgoing.FrameOffsets.y";
  frame.ScrollHeightVarPath = "DisplaySettings.Outgoing.ScrollHeight";
  frame.DisplayType = MikSBT.DISPLAYTYPE_OUTGOING;

  -- Set the tooltip for the frame.
  frame.tooltipText = MikSBTOpt.FRAMES[FRAME_INDEX_OUTGOING_SCROLL_AREA_MOVER].Tooltip;
 end

 -- Show the scroll area mover.
 MikSBTOpt.ShowScrollAreaMover();
end


-- **********************************************************************************
-- Setup and show the notification scroll area mover.
-- **********************************************************************************
function MikSBTOpt.ShowNotificationScrollAreaMover()
 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Set the variable paths for the x and y offsets.
  frame.XOffsetVarPath = "DisplaySettings.Notification.FrameOffsets.x";
  frame.YOffsetVarPath = "DisplaySettings.Notification.FrameOffsets.y";
  frame.ScrollHeightVarPath = "DisplaySettings.Notification.ScrollHeight";
  frame.DisplayType = MikSBT.DISPLAYTYPE_NOTIFICATION;

  -- Set the tooltip for the frame.
  frame.tooltipText = MikSBTOpt.FRAMES[FRAME_INDEX_NOTIFICATION_SCROLL_AREA_MOVER].Tooltip;
 end

 -- Show the scroll area mover.
 MikSBTOpt.ShowScrollAreaMover();
end


-- **********************************************************************************
-- Save the position of the scroll area mover to the appropriate scroll area.
-- **********************************************************************************
function MikSBTOpt.SaveScrollAreaMoverPosition()
 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  local uiParentX, uiParentY = UIParent:GetCenter();
  local frameX = frame:GetCenter();
  local frameY = frame:GetBottom();

  -- Calculate the x offset and set it.
  local xOffset = frameX - uiParentX;
  if (frame.XOffsetVarPath) then
   MikSBT.SetOptionFromVarPath(frame.XOffsetVarPath, xOffset);
  end

  -- Calculate the y offset and set.
  local yOffset = math.ceil(frameY - uiParentY);
  if (frame.YOffsetVarPath) then
   MikSBT.SetOptionFromVarPath(frame.YOffsetVarPath, yOffset);
  end

  -- Reposition the animation frames.
  MikSBT.RepositionAnimationFrames();
 end
end


-- **********************************************************************************
-- Previews a test message in the new scroll area position.
-- **********************************************************************************
function MikSBTOpt.ScrollAreaMoverPreviewOnClick()

 -- Save the position of the scroll are mover.
 MikSBTOpt.SaveScrollAreaMoverPosition();

 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Display the preview message.
  MikSBT.DisplayMessage(MikSBTOpt.MSG_SCROLL_AREA_PREVIEW_TEXT, frame.DisplayType, false);
 end 
end


-- **********************************************************************************
-- Called the save button on the scroll area mover is clicked.
-- **********************************************************************************
function MikSBTOpt.ScrollAreaMoverSaveOnClick()
 -- Save the position of the scroll area mover.
 MikSBTOpt.SaveScrollAreaMoverPosition();

 -- Get the options frame and make sure it exists.
 local optionsFrame = getglobal(OPTIONS_FRAME_NAME);
 if (optionsFrame) then
  -- Show the options frame.
  optionsFrame:Show();
 end

 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Hide the scroll area mover frame.
  frame:Hide();
 end
end


-- **********************************************************************************
-- Cancels the changes made to the scroll area mover.
-- **********************************************************************************
function MikSBTOpt.ScrollAreaMoverCancelOnClick()
 -- Get the scroll area mover frame and make sure it exists.
 local frame = getglobal(SCROLL_AREA_MOVER_FRAME_NAME);
 if (frame) then
  -- Set the x offset back to the saved value.
  if (frame.SavedXOffset ~= nil and frame.XOffsetVarPath) then
   MikSBT.SetOptionFromVarPath(frame.XOffsetVarPath, frame.SavedXOffset);
  end

  -- Set the y offset back to the saved value.
  if (frame.SavedYOffset ~= nil and frame.YOffsetVarPath) then
   MikSBT.SetOptionFromVarPath(frame.YOffsetVarPath, frame.SavedYOffset);
  end

  -- Get the options frame and make sure it exists.
  local optionsFrame = getglobal(OPTIONS_FRAME_NAME);
  if (optionsFrame) then
   -- Show the options frame.
   optionsFrame:Show();
  end

  -- Hide the scroll area mover frame.
  frame:Hide();
 end 
end


-------------------------------------------------------------------------------------
-- Populate Profile Functions.
-------------------------------------------------------------------------------------

-- **********************************************************************************
-- Populates the options from the currently selected profile.
-- **********************************************************************************
function MikSBTOpt.PopulateProfileOptions()
 -- Hide the font settings frame when profile options are populated.
 getglobal(FONT_SETTINGS_FRAME_NAME):Hide();

 -- Set the Animation Step slider value.
 MikSBTOpt.PopulateSlider(SLIDER_INDEX_ANIMATION_STEP);

 -- Set the Show Partial Events checkbox.
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_SHOW_PARTIAL_EFFECTS);

 -- Set the Use Sticky Crits checkbox.
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_USE_STICKY_CRITS);

 -- Set the Show Overheals checkbox.
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_SHOW_OVERHEALS);

 -- Set the Show Game Damage checkbox and set it appropriately.
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_SHOW_GAME_DAMAGE);
 MikSBT.SetOptionShowGameDamage(MikSBT_Save.Profiles[MikSBT_Save.CurrentProfile].ShowGameDamage);

 

 -- Set the scroll area show events checkboxes to the values in the profile.
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_SHOW_INCOMING_EVENTS);
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_SHOW_OUTGOING_EVENTS);
 MikSBTOpt.PopulateCheckbox(CHECKBOX_INDEX_SHOW_NOTIFICATION_EVENTS);

 -- Set the text aligns to the values in the profile.
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_INCOMING_TEXT_ALIGN);
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_OUTGOING_TEXT_ALIGN);
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_NOTIFICATION_TEXT_ALIGN);

 -- Set the animation styles to the values in the profile.
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_INCOMING_ANIMATION_STYLE);
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_OUTGOING_ANIMATION_STYLE);
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_NOTIFICATION_ANIMATION_STYLE);

 -- Set the scroll directions to the values in the profile.
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_INCOMING_SCROLL_DIRECTION);
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_OUTGOING_SCROLL_DIRECTION);
 MikSBTOpt.PopulateDropdown(DROPDOWN_INDEX_NOTIFICATION_SCROLL_DIRECTION);

 -- Set the scroll height sliders to the values in the profile.
 MikSBTOpt.PopulateSlider(SLIDER_INDEX_INCOMING_SCROLL_HEIGHT);
 MikSBTOpt.PopulateSlider(SLIDER_INDEX_OUTGOING_SCROLL_HEIGHT);
 MikSBTOpt.PopulateSlider(SLIDER_INDEX_NOTIFICATION_SCROLL_HEIGHT);

 -- Populate the incoming, outgoing, and notification event settings.
 MikSBTOpt.PopulateIncomingEventSettings();
 MikSBTOpt.PopulateOutgoingEventSettings();
 MikSBTOpt.PopulateNotificationEventSettings();
end